/*:VRX         eCSUpdt
*/
Ecsupdt:
/* Rexx routine to update eCS with IBM Fixpacks
   and other updates - Chuck McKinnis */

version_info = 'Version 1.08, Build 2001.12.06'

Call Rxfuncadd 'SysLoadFuncs','Rexxutil','SysLoadFuncs'
Call SysLoadFuncs

our_dir = Directory()
If Lastpos('\', our_dir) = Length(our_dir) Then
   our_prog = our_dir || 'ecsupdt.cmd'
Else
   our_prog = our_dir || '\ecsupdt.cmd'
our_drive = Filespec('d', our_prog)
our_path = our_drive || Filespec('p', our_prog)
boot_drive = SysBootDrive()

Parse Upper Arg run_parm .

/* check for required files */
log_parms = 'verbose log_out log_file log_lines.'
log_lines.0 = 0

If \Check_for_files() Then
   Return 0

If run_parm <> 'RUN' Then
   Do
      pmrexx = SysSearchPath('path', 'pmrexx.exe')
      If SysCreateObject('WPProgram', 'eCS Fixtool', '<WP_DESKTOP>', ,
         'OBJECTID=<ECSUPDT_CMD>;', 'R') Then
         Do
            If SysSetObjectData('<ECSUPDT_CMD>', ,
               'EXENAME=' || pmrexx || ';STARTUPDIR=' || our_dir || ';' ,
               || 'PARAMETERS=' || '"' || our_prog || '"' 'RUN;') Then
               Do
                  Say 'The eCS Fixtool object has been created on your desktop'
                  Call SysSetObjectData '<ECSUPDT_CMD>', 'OPEN=DEFAULT;CCVIEW=NO;'
                  Return 0
               End
            Else
               Do
                  Say 'Unable to update eCS Fixtool object'
                  Return 0
               End
         End
      Else
         Do
            Say 'Unable to create eCS Fixtool object'
            Return 0
         End
   End

If Initialize() Then
   If Locate_target_drive() Then
   If Misc_updates() Then
   If Update_smp() Then
   If Locate_fixpack() Then
   If Build_response() Then
   If Run_fservice() Then
   If Update_config_sys() Then
   If Apply_ecspatch('SYS') Then
   If Apply_ecspatch('DLG') Then
   If Apply_ecspatch('THM') Then
   If Apply_ecspatch('FIX') Then
   Do
      If theme_upd & SysIni(target_os2ini, 'ThemeManager', 'CurrentTheme') = 'ERROR:' Then
         Call SysIni target_os2ini, 'ThemeManager', 'CurrentTheme', theme_curr
      If \service_bypass | \system_fix Then
         Do
            Call Clean_locklist target_drive
            Call Update_ibmcsflk target_drive
            Call Rx_sayout ''
            Call Rx_sayout 'You will need to re-boot volume' Rx_driveinfo(target_drive, '-vn') 'on drive' target_drive
            Call Rx_sayout 'to activate the applied service and/or updates.', 1
         End
      Else
         Call Rx_sayout 'Service processing was bypassed for drive' target_drive || '.'
   End

If Rx_fileexists(ecswork_dir) Then
   Do
      If SysDestroyObject(ecswork_dir) Then
         Call Rx_sayout 'Removed work directory' ecswork_dir || '.'
      Else
         Call Rx_sayout 'Unable to remove work directory' ecswork_dir || '.'
   End

log_out = 0
Call Stream log_file, 'c', 'close'

Call Rx_sayout 'If you would like to apply another update'
Call Rx_sayout 'please enter "more", or press "Enter" to exit.', 1
Parse Upper Pull ans .
If ans = 'MORE' Then
   Call Ecsupdt 'run'
Call SysDestroyObject '<ECSUPDT_CMD>'

Return 0

/*:VRX         Apply_ecspatch
*/
Apply_ecspatch:
/* setup for eCS patches */

Parse Arg patch_type
Parse Value Find_locklist(target_drive) With lock_exists lock_list lock_dir .
If skip Then
   Parse Value('1 0') With apply service_bypass .

If ((apply | backout) & \service_bypass) | ((apply | backout) & \system_fix) Then
   Do
      If lock_exists & lock_dir = '' Then
         Do
            Call Rx_sayout 'Unable to locate locked file directory.'
            Return 0
         End
      lock_base = lock_dir || '\FIX'
      If apply Then
         info_msg = 'applied'
      Else
         info_msg = 'backed out'
      Call Rx_sayout ''
      Select
         When patch_type = 'SYS' Then
            Do
               If system_fix Then
                  Do
                     Call Rx_sayout 'System updates from Serenity Systems will now be' info_msg || '.'
                     finished = 'system updates from Serenity Systems'
                  End
               Else
                  Return 1
            End
         When patch_type = 'DLG' Then
            Do
               If system_fix Then
                  Do
                     Call Rx_sayout 'Dialog Enhancer updates from Serenity Systems will now be' info_msg || '.'
                     finished = 'Dialog Enhancer updates from Serenity Systems'
                  End
               Else
                  Return 1
            End
         When patch_type = 'THM' Then
            Do
               If system_fix Then
                  Do
                     Call Rx_sayout 'Theme Manager updates from Serenity Systems will now be' info_msg || '.'
                     finished = 'Theme Manager updates from Serenity Systems'
                  End
               Else
                  Return 1
            End
         When patch_type = 'FIX' & \system_fix Then
            Do
               Call Rx_sayout valid_desc 'will now be' info_msg || '.'
               finished = valid_desc
            End
         Otherwise
            Return 1
      End
      If (apply | \system_fix) & \backout Then
         Do
            If patch_type = 'FIX' | skip Then
               Do
                  Call Rx_sayout 'Press enter to continue or enter any other response to bypass the updates.', 1
                  Parse Upper Pull ans .
               End
            Else
               ans = ''
         End
      Else
         ans = ''

      If ans = '' Then
         Do
            Call Stream sys_lvl, 'c', 'close'
            Call Stream sys_lvl, 'c', 'open read'
            new_updt = Charin(sys_lvl, 48, 4)
            Call Stream sys_lvl, 'c', 'close'
            patch_list = our_path || 'ecsnull.lst'
            dlg_list = our_path || 'dlgnull.dat'
            lock_list_updates. = ''
            lock_list_updates.0 = 0
            Select
               When patch_type = 'SYS' Then
                  Do
                     If smp_kernel | install_smp Then
                        patch_list = our_path || 'acp' || new_updt || '.lst'
                     Else
                        patch_list = our_path || 'mcp' || new_updt || '.lst'
                  End
               When patch_type = 'DLG' Then
                  Do
                     If dlg_inst Then
                        Do
                           Select
                              When apply & system_fix Then
                                 dlg_list = our_path || 'dlg' || new_updt || '.dat'
                              When backout & system_fix Then
                                 dlg_list = our_path || 'dlg' || Substr(sys_base, 4) || '.dat'
                              Otherwise
                                 Do
                                    Call Rx_sayout 'Error in determining the Dialog Enhancer control list.'
                                    Return 1
                                 End
                           End
                           Call Backup_dlg(dlg_list)
                        End
                     Else
                        Do
                           Call Rx_sayout 'The Dialog Enhancer modules are not currently installed'
                           Call Rx_sayout 'on drive' target_drive 'and the updates will not be applied.', 1
                           Return 1
                        End
                     patch_list = Build_dlg_list(dlg_list)
                  End
               When patch_type = 'THM' Then
                  Do
                     If theme_upd Then
                        Do
                           Select
                              When apply & system_fix Then
                                 theme_zip = theme_dir || '\thm' || new_updt || '.zip'
                              When backout & system_fix Then
                                 theme_zip = theme_dir || '\thm' || Substr(sys_base, 4) || '.zip'
                              Otherwise
                                 Do
                                    Call Rx_sayout 'Error in determining the Theme Manager control list.'
                                    Return 1
                                 End
                           End
                           xrc = Build_thm_list(theme_zip)
                           If xrc Then
                              Do
                                 Call Rx_sayout 'The set of DLLs used by Theme Manager has been updated for drive' target_drive || '.'
                                 Call Rx_sayout 'Application of Theme Manager updates from Serenity Systems is complete.'
                              End
                           Else
                              Do
                                 Call Rx_sayout 'Application of Theme Manager updates from Serenity Systems failed.'
                              End
                           Return xrc
                        End
                     Else
                        Do
                           Call Rx_sayout 'Theme Manager is not currently installed on', 1
                           Call Rx_sayout 'drive' target_drive 'and the updates will not be applied', 1
                           Return 1
                        End
                  End
               When patch_type = 'FIX' Then
                  Do
                     patch_list = ecsapar_lst
                  End
               Otherwise
                  Do
                     Call Rx_sayout 'Invalid option <' || patch_type || '> passed to eCS update routine.'
                     Call Rx_sayout 'The eCS updates will not be applied.', 1
                     Return 1
                  End
            End
            If Rx_fileexists(patch_list) Then
               Do
                  xrc = Read_patches(patch_list)
                  patch_list = Translate(patch_list)
                  Do i = 1 To patch_list.0
                     Parse Var patch_list.i patch_action source_file target_file
                     Select
                        When patch_action = 'COPY' Then
                           Do
                              If lock_exists Then
                                 Do
                                    target_lock = Check_lock_list(target_file)
                                    /* not in lock list */
                                    If target_lock = target_file Then
                                       Do
                                          /* update target file name */
                                          target_lock = lock_base || Filespec('p', target_file) || Filespec('n', target_file)
                                          /* create lock directory for file */
                                          Call Rx_makepath lock_base || Filespec('p', target_file)
                                          /* add lock list entry */
                                          x = lock_list_updates.0 + 1
                                          lock_list_updates.x = 'MOVE' target_lock target_file
                                          lock_list_updates.0 = x
                                       End
                                    /* in lock list already so replace file */
                                    xrc = Rx_repmod(source_file, target_lock, '-qf')
                                 End
                              Else
                                 xrc = Rx_repmod(source_file, target_file, '-qf')
                           End
                        When Abbrev(patch_list.i, 'CALL') Then
                           Interpret "'"|| patch_list.i || "'"
                        When patch_action = 'PATCH' Then
                           xrc = Rx_addresscmd(patch_list.i, '-q')
                        When Abbrev(patch_list.i, 'ERASE') Then
                           Call SysFileDelete Subword(patch_list.i, 2)
                        When Abbrev(patch_list.i, 'ATTRIB') Then
                           Call Rx_addresscmd patch_list.i
                        When Abbrev(patch_list.i, 'NOTE') ,
                           | Abbrev(patch_list.i, 'PAUSE') Then
                           Do
                              Call Rx_sayout Subword(patch_list.i, 2)
                              If Abbrev(patch_list.i, 'PAUSE') Then
                                 Parse Pull .
                           End
                        Otherwise
                           Nop
                     End
                  End
                  If lock_list_updates.0 <> 0 Then
                     Do
                        Parse Value Rx_findlines('IBMCSFLK.BLK', lock_list) With ,
                           line_no . '|' .
                        If line_no = 0 Then
                           line_no = 3
                        Else
                           line_no = line_no + 1
                        file_lines.0 = 0
                        Do x = 1 To lock_list_updates.0
                           Parse Var lock_list_updates.x line_data
                           xrc = Rx_updtline(line_no, line_data, lock_list, 'insert', '-qe')
                           If x = lock_list_updates.0 Then
                              xrc = Rx_updtline(-1, , lock_list, , '-cqe')
                        End
                     End
                  If patch_type = 'FIX' Then
                     Do
                        Parse Value Filespec('n', patch_list) With patch_name '.' patch_ext
                        patch_date_time = Left(Date('S'), 4) || '.' || Substr(Date('S'), 5, 2) || '.' || Substr(Date('S'), 7) Time()
                        Call SysIni ecsupdt_ini, patch_name, patch_list, patch_date_time
                        If apply Then
                           Do
                              Call SysIni ecsupdt_ini, patch_name, patch_name || '.BOL', 'DELETE:'
                           End
                        Else
                           Do
                              Call SysIni ecsupdt_ini, patch_name, patch_name || '.LST', 'DELETE:'
                              Call SysIni ecsupdt_ini, patch_name, patch_name || '.CDR', 'DELETE:'
                              If Rx_fileexists(backup_name) Then
                                 Call SysFileDelete backup_name
                           End
                     End
                  Call Rx_sayout 'Application of' finished 'is complete.'
                  Drop finished
                  xrc = 1
               End
            Else
               Do
                  Call Rx_sayout patch_list 'not found, no updates to be made.'
                  xrc = 1
               End
         End
      Else
         Do
            Call Rx_sayout 'Bypassing application of' finished || '.'
            Drop finished
            xrc = 1
         End
   End
Else
   xrc = 1

Return xrc

/*:VRX         Backup_dlg
*/
Backup_dlg: Procedure Expose (log_parms) file_lines. dlgback_dir lock_list lock_exists target_drive

Parse Arg dlg_list
dlg_list_name = Filespec('n', dlg_list)
dlg_back = dlgback_dir || '\dlg' || Substr(dlg_list_name, 4, 4) || '.zip'
dlg_list = Filespec('d', dlg_list) ,
   || Filespec('p', dlg_list) ,
   || 'dlg' || Substr(dlg_list_name, 4, 4) || '.upd'
If Rx_fileexists(dlg_list) ,
   & Rx_fileexists(dlgback_dir) ,
   & \Rx_fileexists(dlg_back) Then
   Do
      file_lines. = ''
      If Rx_readlines(dlg_list) Then
         Do
            Call Rx_sayout 'Backing up modules to be updated by Dialog Enhancer, please wait.'
            Call Rx_sayout 'Backing up Dialog Enhancer modules from', , 1
            Call Rx_sayout dlg_list 'to' dlg_back || '.', 1, 1
            Do x = 1 To file_lines.0
               If Abbrev(file_lines.x, ';') | (Space(file_lines.x) = '') Then
                  Iterate x
               Parse Var file_lines.x source_file .
               dlg_file = target_drive || Space(source_file)
               dlg_file_name = Filespec('n', dlg_file)
               dlg_lock = Check_lock_list(dlg_file)
               If dlg_lock <> dlg_file Then
                  Do
                     xrc = Rx_repmod(dlg_lock, dlgback_dir || '\' || dlg_file_name, '-q')
                     dlg_file = dlgback_dir || '\' || dlg_file_name
                  End
               If Rx_fileexists(dlg_file) Then
                  Call Rx_addresscmd '@zip -qjo' dlg_back dlg_file
               If Rx_fileexists(dlgback_dir || '\' || Filespec('n', dlg_file)) Then
                  Call SysFileDelete dlgback_dir || '\' || Filespec('n', dlg_file)
            End
         End
   End

Return 1

/*:VRX         Backup_ecs_code
*/
Backup_ecs_code:

Parse Arg source_drive .
If source_drive = '' Then
   source_drive = target_drive
If \Rx_fileexists(ecsback_dir || '\krnlmcp.zip') Then
   Do
      If Rx_fileexists(source_drive || '\os2krnl') Then
         xrc = Rx_addresscmd('@zip -oqDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\os2krnl') ,
         & Rx_addresscmd('@zip -oqDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\os2ldr') ,
         & Rx_addresscmd('@zip -oqDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\os2\sesdd32.*') ,
         & Rx_addresscmd('@zip -oqDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\os2\dll\doscall1.dll')
      Else
         xrc = Rx_addresscmd('@zip -jqoDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\ecs\os2root\os2krnl') ,
         & Rx_addresscmd('@zip -jqoDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\ecs\os2root\os2ldr') ,
         & Rx_addresscmd('@zip -oqDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\os2\sesdd32.*') ,
         & Rx_addresscmd('@zip -oqDS' ecsback_dir || '\krnlmcp.zip' source_drive || '\os2\dll\doscall1.dll')
   End
If xrc & \Rx_fileexists(ecsback_dir || '\pmdfmcp.zip') Then
   xrc = Rx_addresscmd('@zip -oqDS' ecsback_dir || '\pmdfmcp.zip' source_drive || '\os2\pdpsi\pmdf\*')
If xrc & \Rx_fileexists(ecsback_dir || '\tracemcp.zip') Then
   xrc = Rx_addresscmd('@zip -oqDS' ecsback_dir || '\tracemcp.zip' source_drive || '\os2\system\trace\*')
If xrc Then
   xrc = Rx_addresscmd('@attrib +r' ecsback_dir || '\*.zip', '-q')
If xrc Then
   If Rx_fileexists(target_drive || '\ibmlan\netprog') Then
   Do
      /*
            If source_drive = target_drive Then
               Do
                  If \Rx_fileexists(target_drive || '\ibmlan\netprog\netwksta.ecs') Then
                     xrc = Rx_repmod(target_drive || '\ibmlan\netprog\netwksta.200', ,
                     target_drive || '\ibmlan\netprog\NETWKSTA.ECS')
               End
            Else
               Do
                  If \Rx_fileexists(target_drive || '\ibmlan\netprog\netwksta.ecs') Then
                     Do
                        If Rx_addresscmd('@unzip -joC' source_drive || '\zipfiles\ibmlan.zip' ,
                           'netwksta.200 -d' target_drive) Then
                           Do
                              xrc = Rx_repmod(target_drive || '\netwksta.200', ,
                                 target_drive || '\ibmlan\netprog\NETWKSTA.ECS')
                              If Rx_fileexists(target_drive || '\netwksta.200') Then
                                 Call SysFileDelete target_drive || '\netwksta.200'
                           End
                     End
               End
      */
   End
If xrc = 0 Then
   Call Rx_sayout 'Backup of current system failed.'

Return xrc

/*:VRX         Backup_thm
backup the current theme manager dlls
*/
Backup_thm: Procedure Expose (log_parms) theme_dir
save_trace = Trace('r')
Parse Arg theme_zip

If \Rx_fileexists(theme_zip) Then
   Do
      Call Rx_sayout 'Backing up current Theme Manager DLLs, please wait.'
      xrc = Rx_addresscmd('@zip -joq' theme_zip theme_dir || '\*.dll')
   End
Trace(save_trace)
Return 1

/*:VRX         Build_dlg_list
*/
Build_dlg_list: Procedure Expose (log_parms) file_lines. dlgback_dir ,
   our_path ecswork_dir target_drive apply backout dlgwork_dir

Parse Arg dlg_list
dlg_list_name = Filespec('n', dlg_list)
patch_list = our_path || 'ecsnull.lst'
If apply Then
   dlg_zip = 'dlg' || Substr(dlg_list_name, 4, 4) || '?.zip'
If backout Then
   dlg_zip = dlgback_dir || '\dlg' || Substr(dlg_list_name, 4, 4) || '.zip'
dlgwork_dir = SysTempFileName(ecswork_dir || '\DLG????')
dlg. = ''
dlg.0 = 0
If Rx_readlines(dlg_list) Then
   Do
      Do x = 1 To file_lines.0
         If Abbrev(file_lines.x, '//') | (Space(file_lines.x) = '') Then
            Iterate x
         Parse Var file_lines.x source_name target .
         target = '.' || Space(target) || '\*'
         source_name = dlgwork_dir || '\' || Space(source_name)
         y = dlg.0 + 1
         dlg.y = 'copy' source_name target
         dlg.0 = y
      End
      If dlg.0 <> 0 Then
         Do
            Call Rx_makepath dlgwork_dir
            Call Rx_addresscmd '@copy "' || dlg_list || '"' dlgwork_dir, '-q'
            patch_list = Left(dlg_list_name, Lastpos('.', dlg_list_name)) || 'lst'
            patch_list = dlgwork_dir || '\' || patch_list
            file_lines. = ''
            file_lines.1 = "address cmd 'unzip -joqq" dlg_zip "-d" dlgwork_dir "2>&1|RXQUEUE'"
            file_lines.0 = 1
            Do x = 1 To dlg.0
               y = file_lines.0 + 1
               file_lines.y = dlg.x
               file_lines.0 = y
            End
            y = file_lines.0 + 1
            file_lines.y = 'copy' dlgwork_dir || '\' || dlg_list_name '.\ecs\install\dlg_inst.dat'
            file_lines.0 = y
         End
      Call Rx_writelines patch_list
   End

Return patch_list

/*:VRX         Build_response
*/
Build_response:
/* build service response file */

replace_newer = 0

If \(apply | backout) | \system_fix Then
   Return 1

If smp_upgrade & (sys_curr <> fix_name) Then
   Do
      Call Rx_sayout 'Fixpack' fix_name 'cannot be used to do' action_desc 'processing'
      Call Rx_sayout 'on your current system at fixpack level' sys_curr || '.', 1
      Return 0
   End

Call Rx_sayout 'Building service response file.'
service_exe = srv_path || '\' || service_name
response_file = srv_path || '\RESPONSE.FIL'

If install_smp & \archived Then
   Do
      Call Rx_sayout 'eCS SMP support cannot be installed with service'
      Call Rx_sayout 'processing on a system without an archive.', 1
      Return 0
   End

Call Rx_sayout ''
Call Rx_sayout 'IBM service processing will, by default, ask if newer files'
Call Rx_sayout 'should be replaced.  If you want service processing to replace'
Call Rx_sayout 'all files without prompting, you will need to reply "Yes" here.'
Call Rx_sayout 'Do you want to replace newer files without prompting?'
Call Rx_sayout 'Please reply "Yes" or "No" (default = No)'
Parse Upper Pull ans
If Abbrev(ans, 'Y') Then
   replace_newer = 1
If \replace_newer & install_smp & apply Then
   Do
      Call Rx_sayout 'The SMP update will result in a large number of files needing'
      Call Rx_sayout 'to be replaced.  You should reply "Yes" to replace any files'
      Call Rx_sayout 'that are in the' target_drive || '\,' target_drive || '\OS2\PDPSI\PMDF,' target_drive || '\os2\pdpsi\pmdf\warp45_s,'
      Call Rx_sayout 'or' target_drive || '\OS2\SYSTEM\TRACE directories.'
   End

file_lines. = ''
i = 0
If apply Then
   Do
      i = i + 1; file_lines.i = ':LOGFILE' service_log
      If replace_newer Then
         Do
            i = i + 1; file_lines.i = ':FLAGS REPLACE_PROTECTED REPLACE_NEWER EXIT_WHEN_DONE'
         End
      Else
         Do
            i = i + 1; file_lines.i = ':FLAGS REPLACE_PROTECTED EXIT_WHEN_DONE'
         End
      i = i + 1; file_lines.i = ':SOURCE' csf_path
      i = i + 1; file_lines.i = ':SERVICE'
      i = i + 1; file_lines.i = ':SYSLEVEL' sys_lvl
      If \archived Then
         Do
            i = i + 1; file_lines.i = ':ARCHIVE' archive_path
         End
      Else
         Do
            i = i + 1; file_lines.i = ':BACKUP' backup_path
         End
   End
Else
   Do
      If archived Then
         backout_target = 'ARCHIVE'
      Else
         Do
            Call Rx_sayout 'No archive for' fix_level 'found, aborting.'
            Return 0
         End
      i = i + 1; file_lines.i = ':LOGFILE' service_log
      i = i + 1; file_lines.i = ':FLAGS EXIT_WHEN_DONE'
      i = i + 1; file_lines.i = ':SOURCE' csf_path
      i = i + 1; file_lines.i = ':TARGET' backout_target
      i = i + 1; file_lines.i = ':BACKOUT'
      i = i + 1; file_lines.i = ':SYSLEVEL' sys_lvl
   End

i = i + 1; file_lines.i = '1a'x
file_lines.0 = i
If Rx_fileexists(response_file) Then
   Call SysFileDelete response_file
Call Rx_writelines response_file

Return 1

/*:VRX         Build_thm_list
*/
Build_thm_list: Procedure Expose (log_parms) file_lines. our_path ecswork_dir ,
   (theme_mgr) lock_exists lock_list target_drive apply backout
save_trace = Trace('N')

Parse Arg theme_zip

pmmrgres = target_drive || '\os2\dll\pmmrgres.dll'
If \Rx_fileexists(theme_zip) Then
   Do
      xrc = Rx_addresscmd('@unzip -qqjCo' '"' || our_path || 'thmcode.zip" -d' theme_dir)
      /* locate the pmmrgres.dll to be used and copy to work dir */
      pmmrgres = Check_lock_list(pmmrgres)
      Call Rx_repmod pmmrgres, theme_dir || '\pmmrgres.dll', '-qf'
      xrc = Rx_addresscmd('unzip -joq' theme_bmp '-d' theme_dir)
      xrc = Rx_addresscmd('unzip -joq' theme_res '-d' theme_dir)

      /* update resources */
      Call Rx_sayout 'Updating Theme Manager resources, please wait.'
      Call Themeupd theme_dir
      /* delete so that fixpacks will not update */
      Call SysFileDelete theme_dir ||'\pmmrgres.dll'
      /* clean out the files we unzipped */
      Address cmd '@attrib -r -s -h' theme_dir || '\*.*'
      Address cmd '@erase' theme_dir || '\rc.exe > nul'
      Address cmd '@erase' theme_dir || '\rcpp.err > nul'
      Address cmd '@erase' theme_dir || '\rcpp.exe > nul'
      Address cmd '@erase' theme_dir || '\rdcn.cmd > nul'
      Address cmd '@erase' theme_dir || '\rdcpp.exe > nul'
      Address cmd '@erase' theme_dir || '\resmgrn.cmd > nul'
      Address cmd '@erase' theme_dir || '\themeupd.cmd > nul'
   End
Else
   xrc = Rx_addresscmd('unzip -joq' theme_zip '-d' theme_dir)

/* update pmmrgres.dll with updated theme manager dll */
Parse Value Rx_findlines(theme_curr, theme_idx) With ,
   theme_line_no theme_line '|' .
If theme_line_no <> 0 Then
   Do
      Parse Var theme_line '' . '' theme_dll '' .
      theme_dll = Strip(theme_dll)
      xrc = Rx_repmod(theme_dir || '\' || theme_dll, pmmrgres, '-qf')
   End
/* back up current theme manager updates if necessary */
If \Rx_fileexists(theme_zip) Then
   Call Rx_addresscmd '@zip -joq' theme_zip theme_dir || '\*.dll'
/* clean out the bmp and res files */
If Rx_fileexists(theme_bmp) Then
   Call Rx_addresscmd '@erase' theme_dir || '\*.bmp > nul', '-q'
If Rx_fileexists(theme_res) Then
   Call Rx_addresscmd '@erase' theme_dir || '\*.res > nul', '-q'
Trace(save_trace)
Return xrc

/*:VRX         Check_system
*/
Check_system: Procedure Expose (log_parms) file_lines. boot_drive
save_trace = Trace('N')
Parse Arg test_drive

test_config = Translate(Stream(test_drive || '\config.sys', 'c', 'query exists'))
test_os2ini = test_drive || '\os2\os2.ini'
test_os2sysini = test_drive || '\os2\os2sys.ini'
updates. = ''
updates.0 = 0

/* make sure that ini files are writeable */
Call SysFileTree test_os2ini, 'test.', 'fo', '*****', '**---'
Call SysFileTree test_os2sysini, 'test.', 'fo', '*****', '**---'

/* updates for boot drive only */
If test_drive = boot_drive Then
   Do

      /* If necessary, REM out SET DESKTOP= parm in CONFIG.SYS */
      Parse Value Rx_findlines('set desktop=', test_config) With ,
         line_no delete_line '|' .
      line_no = Strip(line_no)
      If line_no <> 0 Then
         Do
            If SysIni(test_os2ini, 'PM_Workplace:Location', '<WP_DESKTOP>') <> 'ERROR:' ,
               & SysSetObjectData('<WP_DESKTOP>', 'NODELETE=YES;') Then
               Do
                  x = updates.0 + 1
                  updates.x = line_no 'replace' 'REM' delete_line
                  updates.0 = x
                  Call Rx_sayout 'REMmed out SET DESKTOP= in' test_config, 1, 1
               End
         End

      /* update Refresh Removable Media */
      lvm_parms = 'OBJECTID=<LVMREFRESH>;EXENAME=LVM.EXE;PARAMETERS=/RediscoverPRM;ICONFILE=' || boot_drive || '\os2\look.ico' || ';'
      If SysCreateObject('WPProgram', 'Refresh^Removeable Media', '<WP_DRIVES>', lvm_parms, 'U') Then
         Call Rx_sayout 'Updated Refresh Removable Media', 1, 1

      /* fix Debug Window (PM Rexx) parameters */
      If WPToolsQueryObject('<WP_DEBUGWINDOW>', , , 'szSetupString') Then
         Do
            Parse Value '.' szsetupstring With . 'PARAMETERS=' pmrexx_parms ';' .
            pmrexx_parms = Strip(pmrexx_parms)
            If pmrexx_parms = 'REXXTRY \ECS\CMD.EXE' Then
               Do
                  pmrexx_parms = 'PARAMETERS=REXXTRY.CMD;STARTUPDIR=;'
                  If SysSetObjectData('<WP_DEBUGWINDOW>', pmrexx_parms) Then
                     Call Rx_sayout 'Setting the current parameters of PM Rexx to:' pmrexx_parms, 1, 1
               End
         End
   End

/* check and fix PM_SystemFonts entry in OS2.INI if necessary */
If SysIni(test_os2ini, 'PM_SystemFonts', 'ALL:', 'ini_keys.') <> 'ERROR:' Then
   Do x = 1 To ini_keys.0
      test_key = SysIni(test_os2ini, 'PM_SystemFonts', ini_keys.x)
      If (test_key <> 'ERROR:') ,
         & (Substr(test_key, Length(test_key)) <> '00'x) Then
         Call SysIni test_os2ini, 'PM_SystemFonts', ini_keys.x, test_key || '00'x
   End

If test_config = '' Then
   Do
      Trace(save_trace)
      Return 1
   End
Else
   Call Rx_sayout 'Examining' test_config, , 1

/* If necessary, set THREADS=1024 in CONFIG.SYS */
line_no = 0
threads = ''
frc = SysFileSearch('threads=', test_config, 'test.', 'n')
If frc = 0 Then
   Do i = 1 To test.0
      Parse Var test.i test_line_no test_line
      If Abbrev(Strip(Translate(test_line)), 'THREADS') Then
         Do
            line_no = Strip(test_line_no)
            thread_line = Strip(test_line)
            Parse Var thread_line . '=' threads
            threads = Strip(threads)
         End
   End
If line_no = 0 | threads < 1024 Then
   Do
      new_threads = 'THREADS=1024'
      x = updates.0 + 1
      If line_no <> 0 Then
         Do
            updates.x = line_no 'replace' new_threads
            Call Rx_sayout 'Updated' thread_line 'to' new_threads 'in' test_config, 1, 1
         End
      Else
         Do
            updates.x = '1 insert' new_threads
            Call Rx_sayout 'Added' new_threads 'to' test_config, 1, 1
         End
      updates.0 = x
   End

/* If necessary, REM out SET DELDIR= parm in CONFIG.SYS */
Parse Value Rx_findlines('set deldir=', test_config) With ,
   line_no delete_line '|' .
line_no = Strip(line_no)
If line_no <> 0 Then
   Do
      x = updates.0 + 1
      updates.x = line_no 'replace' 'REM' delete_line
      updates.0 = x
      Call Rx_sayout 'REMmed out SET DELDIR= in' test_config, 1, 1
   End

/* If necessary add SET SHELLHANDLESINC=20 to CONFIG.SYS */
Parse Value Rx_findlines('set shellhandlesinc=', test_config) With ,
   line_no . '|' .
line_no = Strip(line_no)
If line_no = 0 Then
   Do
      Parse Value Rx_findlines('set os2_shell=', test_config) With ,
         line_no . '|' rem_line_no .
      line_no = Strip(line_no)
      rem_line_no = Strip(rem_line_no)
      If line_no = 0 & rem_line_no <> 0 Then
         line_no = rem_line_no
      If line_no <> 0 Then
         Do
            line_no = line_no + 1
            x = updates.0 + 1
            updates.x = line_no 'insert' 'SET SHELLHANDLESINC=20'
            updates.0 = x
            Call Rx_sayout 'Added SET SHELLHANDLESINC=20 to' test_config, 1, 1
         End
   End

/* If necessary, add LANPDD and LANVDD device drivers to CONFIG.SYS */
add_lan = 0
If SysFileSearch('[netbeui_nif]', test_drive || '\ibmcom\protocol.ini', 'test.') = 0 Then
   add_lan = (test.0 <> 0)
If \add_lan Then
   If SysFileSearch('[tcpip_nif]', test_drive || '\ibmcom\protocol.ini', 'test.') = 0 Then
   add_lan = (test.0 <> 0)
Parse Value Rx_findlines(test_drive || '\ibmcom\lanmsgdd.os2', test_config) With ,
   line_no . '|' .
If add_lan & line_no <> 0 Then
   Do
      Parse Value Rx_findlines('\ibmcom\protocol\lanvdd.os2', test_config) With ,
         chk_line_no . '|' .
      If Rx_fileexists(test_drive || '\ibmcom\protocol\lanvdd.os2') ,
         & chk_line_no = 0 Then
         Do
            x = updates.0 + 1
            updates.x = line_no 'insert' 'DEVICE=' || test_drive || '\IBMCOM\PROTOCOL\LANVDD.OS2'
            updates.0 = x
            Call Rx_sayout 'Added DEVICE=' || test_drive || '\IBMCOM\PROTOCOL\LANVDD.OS2 to' test_config, 1, 1
         End
      Parse Value Rx_findlines('\ibmcom\protocol\lanpdd.os2', test_config) With ,
         chk_line_no chk_line '|' .
      If Rx_fileexists(test_drive || '\ibmcom\protocol\lanpdd.os2') ,
         & chk_line_no = 0 Then
         Do
            x = updates.0 + 1
            updates.x = line_no 'insert' 'DEVICE=' || test_drive || '\IBMCOM\PROTOCOL\LANPDD.OS2'
            updates.0 = x
            Call Rx_sayout 'Added DEVICE=' || test_drive || '\IBMCOM\PROTOCOL\LANPDD.OS2 to' test_config, 1, 1
         End
   End

If updates.0 > 1 Then
   Do
      /* make sure that highest line number is updated first */
      changed = 1
      Do x = 1 To updates.0 While changed
         y = x + 1
         If Word(updates.x, 1) < Word(updates.y, 1) Then
            Do
               save_update = updates.x
               updates.x = updates.y
               updates.y = save_update
               x = 0
            End
         Else
            If x = updates.0 Then
            changed = 0
      End
   End

xrc = 1
file_lines.0 = 0
Do x = 1 To updates.0
   Parse Var updates.x line_no action line
   xrc = Rx_updtline(line_no, line, test_config, action, '-q')
   If action = 'insert' Then
      action_type = 'inserted'
   Else
      action_type = 'replaced'
   xrc = Rx_updtline(line_no, 'REM following line' action_type 'by eCSUpdt', test_config, 'insert', '-q')
   If x = updates.0 Then
      xrc = Rx_updtline(-1, , test_config, , '-cq')
End
Trace(save_trace)
Return xrc

/*:VRX         Check_ecs_backup
*/
Check_ecs_backup:

xrc = 0
If Rx_fileexists(ecsback_dir || '\krnlmcp.zip') ,
   & Rx_fileexists(ecsback_dir || '\pmdfmcp.zip') ,
   & Rx_fileexists(ecsback_dir || '\tracemcp.zip') Then
   Do
      xrc = 1
      /*
            If Rx_fileexists(target_drive || '\ibmlan\netprog') Then
               xrc = Rx_fileexists(target_drive || '\ibmlan\netprog\netwksta.ecs')
      */
   End
If \xrc Then
   Do
      Call Rx_sayout 'The necessary backup files in' ecsback_dir 'were not found.'
      If sys_curr = sys_base Then
         Do
            Call Rx_sayout 'Building the backup files from' our_path || 'os2root.', 1
            If Backup_ecs_code(our_path || 'os2root') Then
               xrc = Check_ecs_backup()
         End
      If \xrc Then
         Call Rx_sayout 'Ecsupdt will not be able to uninstall SMP on drive' target_drive || '.', 1
   End

Return xrc

/*:VRX         Check_for_files
*/
Check_for_files: Procedure Expose (log_parms) boot_drive
save_trace = Trace('N')

/* make sure that ini files are writeable */
Call SysFileTree boot_drive || '\os2\os2*.ini', 'test.', 'fo', '*****', '**---'

req_files. = ''
x = 0
/* working files */
x = x + 1; req_files.x = 'ecsupdt.cmd'
x = x + 1; req_files.x = 'header.txt'
x = x + 1; req_files.x = 'readme.txt'
/* utility modules */
x = x + 1; req_files.x = 'checksum.exe'
x = x + 1; req_files.x = 'lvm.exe'
x = x + 1; req_files.x = 'pmrexx.exe'
x = x + 1; req_files.x = 'repmod.exe'
x = x + 1; req_files.x = 'unzip.exe'
x = x + 1; req_files.x = 'wptools.dll'
x = x + 1; req_files.x = 'zip.exe'
/* control files */
x = x + 1; req_files.x = 'acpc001.lst'
x = x + 1; req_files.x = 'acpc001.pat'
x = x + 1; req_files.x = 'dlg4501.dat'
x = x + 1; req_files.x = 'dlgc001.dat'
x = x + 1; req_files.x = 'dlgc001.upd'
x = x + 1; req_files.x = 'mcp4501.lst'
x = x + 1; req_files.x = 'mcpc001.lst'
x = x + 1; req_files.x = 'mcpc001.pat'
req_files.0 = x

/* check that we have all of the files needed */
Do x = 1 To req_files.0
   module_name = Strip(req_files.x)
   find_dll = (Translate(Substr(module_name, Lastpos('.', module_name))) = '.DLL')
   If \find_dll Then
      module_test = SysSearchPath('path', module_name)
   Else
      module_test = Rx_searchlibpath(module_name)
   If module_test <> '' Then
      Iterate x
   Else
      Do
         xrc = SysFileTree(boot_drive || '\' || module_name, 'test.', 'fso')
         If xrc = 0 & test.0 <> 0 Then
            Do
               If \find_dll Then
                  Address cmd '@copy' test.1 boot_drive || '\os2\apps 2>&1|RXQUEUE'
               Else
                  Address cmd '@copy' test.1 boot_drive || '\os2\apps\dll 2>&1|RXQUEUE'
               Call Rx_dequeue
               Iterate x
            End
         Else
            Do
               Say 'Required file' module_name 'was not found in your path'
               Return 0
            End
      End
End

/* if necessary, set object id of desktop to the standard of <WP_DESKTOP> */
desktop_dir = boot_drive || '\desktop'
frc = SysFileTree(desktop_dir, 'test.', 'do')
fix_desktop = 0
If frc = 0 & test.0 <> 0 Then
   Do
      desktop_dir = Translate(test.1)
      /* test the desktop */
      test_os2ini = boot_drive || '\os2\os2.ini'
      If SysIni(test_os2ini, 'PM_Workplace:Location', '<WP_DESKTOP>') = 'ERROR:' Then
         Do
            Call Rx_sayout '<WP_DESKTOP> not found in' test_os2ini
            fix_desktop = 1
         End
      If \SysSetObjectData('<WP_DESKTOP>', 'NODELETE=YES;') Then
         Do
            Call Rx_sayout 'Unable to access <WP_DESKTOP>'
            fix_desktop = 1
         End
      If fix_desktop Then
         Do
            If SysSetObjectData(desktop_dir, 'OBJECTID=<WP_DESKTOP>;') Then
               Call Rx_sayout 'Setting the object parms of' desktop_dir 'to OBJECTID=<WP_DESKTOP>;'
            Else
               Call Rx_sayout 'Unable to set the object parms of' desktop_dir 'to OBJECTID=<WP_DESKTOP>;'
            Call Rx_sayout 'Please shutdown and re-boot drive' boot_drive 'before continuing'
            Return 0
         End
   End

Call Rxfuncadd 'WPToolsLoadFuncs', 'WPTOOLS', 'WPToolsLoadFuncs'
Call WPToolsLoadFuncs

Call Rx_dequeue
Trace(save_trace)
Return 1

Check_lock_list: Procedure Expose lock_exists lock_list
save_trace = Trace('N')
Parse Arg file
If lock_exists Then
   Do
      Parse Value Rx_findlines(file, lock_list) With ,
         file_line_no file_line '|' .
      If file_line_no <> 0 Then
         Do
            Parse Var file_line . file .
            file = Space(file)
         End
   End
Trace(save_trace)
Return file

/*:VRX         Check_syslevel
*/
Check_syslevel: Procedure Expose (log_parms) target_drive ecsback_dir
/* check current syslevel.os2 */
save_trace = Trace('N')
Parse Arg sys_lvl
Call Stream sys_lvl, 'c', 'close'
Call Stream sys_lvl, 'c', 'open read'
sys_curr = Charin(sys_lvl, 48, 4)
sys_base = Charin(sys_lvl, 56, 4)
Call Stream sys_lvl, 'c', 'close'
/* if system has been serviced, don't update */
If sys_curr <> sys_base Then
   Return 1
sys_lvl_length = Stream(sys_lvl, 'c', 'query size')
/* if current size = base size, don't update */
If sys_lvl_length = 165 Then
   Return 1
/* read current syslevel file */
Call Stream sys_lvl, 'c', 'open read'
sys_lvl_org = Charin(sys_lvl, 1, sys_lvl_length)
Call Stream sys_lvl, 'c', 'close'
/* does it reference the target drive? */
If Pos(target_drive, sys_lvl_org) = 0 Then
   Do
      /* target drive not referenced, replace with MCP syslevel */
      Call Rx_sayout 'Replacing invalid' sys_lvl || '.', , 1
      sys_lvl_backup = ecsback_dir || '\SYS' || sys_base || '.ZIP'
      Address cmd '@attrib -r -s -h' sys_lvl
      Call SysSleep 3
      Address cmd '@zip -ojqm' sys_lvl_backup sys_lvl
      Call Stream sys_lvl, 'c', 'close'
      Call Stream sys_lvl, 'c', 'open write'
      xrc = Charout(sys_lvl, Left(sys_lvl_org, 165))
      Call Stream sys_lvl, 'c', 'close'
      xrc = 1
   End
Else
   xrc = 1
Trace(save_trace)
Return xrc

/*:VRX         Get_kernel_info
*/
Get_kernel_info: Procedure Expose (log_parms) lock_list lock_exists
save_trace = Trace('N')
Parse Arg target_drive
If Datatype(lock_exists, 'B') Then
   kernel = Check_lock_list(target_drive || '\os2krnl')
Else
   kernel = target_drive || '\os2krnl'
xrc = 0
Call Rx_addresscmd '@bldlevel' kernel, '-rx'
Do Queued()
   Parse Pull ans
   If \xrc & (Pos('SMP', Translate(ans)) <> 0) Then
      xrc = 1
   If Abbrev(ans, 'Signature:') Then
      Parse Var ans . '@#IBM:' revision '#@' sys_type .
End
revision = Translate(Space(revision) || Space(sys_type))
Trace(save_trace)
Return xrc revision

/*:VRX         Get_smp_runtime
*/
Get_smp_runtime:

If xrc & \Rx_fileexists(target_drive || '\os2\install\smp\@smpdir') Then
   Do
      xrc = Rx_makepath(target_drive || '\os2\install\SMP')
      If xrc & (\Rx_fileexists(target_drive || '\os2\install\smp\os2krnl') ,
         | \Rx_fileexists(target_drive || '\os2\install\smp\os2ldr') ,
         | \Rx_fileexists(target_drive || '\os2\install\smp\doscall1.dll')) Then
         Do
            xrc = Rx_addresscmd('@copy' our_path || 'os2root\os2\install\smp\*.*' target_drive || '\os2\install\smp')
            Call Rx_addresscmd '@attrib -r' target_drive || '\os2\install\smp\*', '-q'
         End
   End
If xrc & \Rx_fileexists(target_drive || '\os2\boot\os2apic.psd') Then
   Do
      xrc = Rx_addresscmd('@copy' our_path || 'os2root\os2\boot\os2apic.psd' target_drive || '\os2\boot')
      Call Rx_addresscmd '@attrib -r' target_drive || '\os2\boot\os2apic.psd', '-q'
   End
If xrc = 0 Then
   Call Rx_sayout 'Copy of SMP files failed.'
Return xrc

/*:VRX         Halt
*/
Halt:

Exit

/*:VRX         Initialize
*/
Initialize:
save_trace = Trace('N')

Call Rx_dequeue

service_name = 'FSERVICE.EXE'
csf_path = ''

xrc = 1

If Wordpos(SysFileSystemType(our_drive), 'HPFS JFS CDFS LAN') = 0 Then
   Do
      Say 'eCSUpdt must run from an HPFS, JFS, CDFS, or LAN drive'
      xrc = 0
   End
Else
   Do
      If Wordpos(SysFileSystemType(our_drive), 'CDFS LAN') <> 0 Then
         Do
            If SysFileSystemType(our_drive) = 'CDFS' Then
               our_cd_label = Rx_driveinfo(our_drive, '-vn')
         End
   End

valid. = ''
valid.0 = 0

theme_mgr = 'theme_dir theme_curr theme_idx theme_bmp theme_res'
verbose = 1
log_out = 0

Call Rx_sayout 'eComStation Fixtool', , -1
Call Rx_sayout version_info, 1, -1
Call Rx_sayout 'Copyright 2001 by Charles H. McKinnis <mckinnis@attglobal.net>', 1, -1
Call Rx_sayout 'Unlimited rights granted to Serenity Systems, Inc', 1, -1
Call Rx_sayout ''

Parse Value '0 0 0 0 0' With apply skip backout commit boot_now .
Parse Value '0 0 0 0 0' With smp_kernel smp_upgrade install_smp uninstall_smp system_fix .

ecs_desc = 'eComStation Operating System'
mcp_comp = '5639A6101'
ecs_pro = 0
pro_desc = 'eComStation Pro Operating System'
acp_comp = '5639A5550'
psd_sys = 'PSD=OS2APIC.PSD'
sys_lvl_ecs = 'SYSLEVEL.ECS'
sys_lvl_path = '\os2\install\'
kernel_updates = 'W41026 SMP1026'
system_updates = kernel_updates 'MCPC001 ACPC001 MCP4501 JFS1026'

/* check the shell */
our_shell = Filespec('n', Translate(Value('COMSPEC', ,'os2environment')))
If our_shell <> 'CMD.EXE' Then
   Do
      Say 'eCSUpdt must run with the standard OS/2 shell, found' our_shell
      xrc = 0
   End

/* check the kernel level */
Parse Value Get_kernel_info(boot_drive) With . boot_revision
If Pos('W4', boot_revision) = 0 Then
   If Pos('SMP', boot_revision) = 0 Then
   Do
      Call Rx_sayout 'The kernel on boot drive' boot_drive 'is not supported by eCS.'
      Call Rx_sayout 'Kernel revision =' boot_revision || '.'
      xrc = 0
   End

/* look for and test fix files */
zip_free = 0
fix_files = 'xr_c001.zip xr_c001_smp.zip ecsc001b.zip ecsc001s.zip'
zips. = ''
zips.0 = 0
If xrc Then
   Do
      zip_free = 3                              /* space for CD files */
      /* look for fix files */
      Do i = 1 To Words(fix_files)
         fix_file = Word(fix_files, i)
         If Rx_fileexists(fix_file) Then
            Do
               fix_name = Filespec('n', fix_file)
               /* test the zip file */
               Address cmd 'zip -T' fix_name '2>&1|RXQUEUE'
               Do Queued()
                  Parse Pull ans
                  If Pos(Translate(fix_name), Translate(ans)) <> 0 Then
                     Do
                        If Pos('OK', ans) <> 0 Then
                           Do
                              j = zips.0 + 1
                              zips.j = fix_name
                              zips.0 = j
                              zip_free = zip_free + Stream(fix_name, 'c', 'query size')
                           End
                        Else
                           Do
                              Call Rx_sayout ans
                              xrc = 0
                           End
                     End
               End
            End
      End
      /* look for ptf files */
      frx = SysFileTree('*.ptf', 'test.', 'fo')
      If frx = 0 Then
         Do i = 1 To test.0
            fix_name = Filespec('n', test.i)
            /* test the zip file */
            Address cmd 'zip -T' fix_name '2>&1|RXQUEUE'
            Do Queued()
               Parse Pull ans
               If Pos(Translate(fix_name), Translate(ans)) <> 0 Then
                  Do
                     If Pos('OK', ans) <> 0 Then
                        Do
                           j = zips.0 + 1
                           zips.j = fix_name
                           zips.0 = j
                           zip_free = zip_free + Stream(fix_name, 'c', 'query size')
                        End
                     Else
                        Do
                           Call Rx_sayout ans
                           xrc = 0
                        End
                  End
            End
         End
   End
zip_free = Format(zip_free / (1024 * 1000), ,0)

/* test for space to unzip fix files */
If Rx_driveinfo(our_drive, '-fr') < zip_free Then
   Do
      Say 'eCSUpdt requires at least' zip_free 'Mb of free space'
      xrc = 0
   End

/* check for fix files not unzipped on LAN or CD */
If zips.0 <> 0 & (Wordpos(SysFileSystemType(our_drive), 'CDFS LAN') <> 0) Then
   Do
      Call Rx_sayout 'The following fix files were found on drive' our_drive
      Call Rx_sayout 'that need to be unzipped before maintenance is applied.'
      Do x = 1 To zips.0
         Call Rx_sayout zips.x, 1
      End
      xrc = 0
   End

/* test and unzip the fix files */
zips_checked = 0
If xrc & zips.0 <> 0 Then
   Do i = 1 To zips.0
      zip_name = Filespec('n', zips.i)
      /* unzip the zip file */
      Call Rx_sayout 'Unzipping' zip_name || '.'
      Address cmd 'unzip -jqo' zip_name '2>&1|RXQUEUE'
      Address cmd 'rename' zip_name '*.BAK'
   End

/* set up to test zip files */
If xrc Then
   zips_checked = (zips.0 = 0)

/* if updated control files exist, unzip them now */
If Rx_fileexists('ecscntl.zip') Then
   Call Rx_addresscmd 'unzip -jqo ecscntl.zip', '-q'

/* check for install info in all *.lst files */
frc = SysFileTree('*.lst', 'lists.', 'fo')
If frc = 0 Then
   Do x = 1 To lists.0
      /* test for install information */
      frc = SysFileSearch('; install_info ', lists.x, 'info.')
      If frc = 0 & info.0 <> 0 Then
         Do y = 1 To info.0
            version_line = Strip(Subword(info.y, 3))
            /* check if already present */
            Do z = 1 To valid.0
               If version_line = valid.z Then
                  Iterate y
            End
            z = valid.0 + 1
            valid.z = version_line
            valid.0 = z
         End
   End

Trace(save_trace)
Return xrc

/*:VRX         Insert_cd
*/
Insert_cd: Procedure Expose (log_parms) our_drive our_dir
/* load cd based on label */
save_trace = Trace('N')
Parse Arg cd_label, message
drive_list = SysDriveMap()
cd_letter = ''
Do i = 1 To Words(drive_list) While cd_letter = ''
   drive = Word(drive_list, i)
   Parse Value SysDriveInfo(drive) With drive_letter . . drive_label .
   If drive_label = cd_label Then
      cd_letter = drive_letter
End
If cd_letter = '' Then
   Do Until (ans <> '' | cd_letter <> '')
      Call Rx_sayout ''
      Call Rx_sayout message 'and press "Enter",'
      Call Rx_sayout 'or enter any other response to exit.', 1
      Parse Upper Pull ans .
      If ans = '' Then
         Do i = 1 To Words(drive_list) While cd_letter = ''
            drive = Word(drive_list, i)
            Parse Value SysDriveInfo(drive) With drive_letter . . drive_label .
            If drive_label = cd_label Then
               cd_letter = drive_letter
         End
   End
If cd_letter = our_drive Then
   Call Directory our_dir
Trace(save_trace)
Return cd_letter

/*:VRX         Load_smp_code
*/
Load_smp_code:

If \apply Then                  /* don't if a fixpack will be applied */
   Do
      /* ses & problem determination code */
      If xrc & Rx_fileexists(our_path || 'smp4501.zip') ,
         & Rx_addresscmd('@attrib -r' target_drive || '\os2\system\trace\*', '-q') ,
         & Rx_addresscmd('@unzip -oq' '"' || our_path || 'smp4501.zip" -d' target_drive || '\') ,
         & Rx_addresscmd('@copy' target_drive || '\os2\pdpsi\pmdf\warp45_s\*.*' target_drive || '\os2\pdpsi\pmdf', '-q') Then
         Do
            If Rx_fileexists(target_drive || '\ibmlan\netprog') Then
               Do
                  /*
                                    If Rx_repmod(target_drive || '\netwksta.200', ,
                                       target_drive || '\ibmlan\netprog\NETWKSTA.200') Then
                                       Call SysFileDelete target_drive || '\netwksta.200'
                  */
                  xrc = 1
               End
            If Rx_fileexists(target_drive || '\os2\pdpsi\pmdf\@warp45s') Then
               xrc = Rx_addresscmd('@erase' target_drive || '\os2\pdpsi\pmdf\@warp45s', '-q') Then
         End
      Else
         Do
            Call Rx_sayout 'Unable to install eCS SMP problem determination code.'
            xrc = 0
         End
      /* SMP running code */
      If xrc Then
         Do
            If Rx_addresscmd('@attrib -r -s -h' target_drive || '\os2krnl', '-q') ,
               & Rx_repmod(target_drive || '\os2\install\smp\os2krnl', target_drive || '\os2krnl', '-f') ,
               & Rx_addresscmd('@attrib +r +s +h' target_drive || '\os2krnl', '-q') ,
               & Rx_addresscmd('@attrib -r -s -h' target_drive || '\os2ldr', '-q') ,
               & Rx_repmod(target_drive || '\os2\install\smp\os2ldr', target_drive || '\os2ldr', '-f') ,
               & Rx_addresscmd('@attrib +r +s +h' target_drive || '\os2ldr', '-q') ,
               & Rx_repmod(target_drive || '\os2\install\smp\doscall1.dll', target_drive || '\os2\dll\doscall1.dll', '-f') Then
               Do
                  Call Rx_sayout 'eCS SMP support has been installed on drive' target_drive || '.'
                  Call Rx_sayout 'Please re-boot drive' target_drive 'to activate.'
                  upd_desc = pro_desc
                  xrc = 1
               End
            Else
               Do
                  Call Rx_sayout 'Unable to install the eCS SMP running code.'
                  upd_desc = ecs_desc
                  xrc = 0
               End
         End
   End
Else
   Do
      Call Rx_sayout 'The running SMP code will be applied from fixpack' valid_fix || '.'
      xrc = 1
   End
Return xrc

/*:VRX         Locate_fixpack
*/
Locate_fixpack:
/* identify fixpack locations */

If \(apply | backout) Then
   Return 1

service. = ''
service.0 = 0

/* locate the downloaded fixpack */
csf_zip = ''
ecsapar_lst = ''
/* look for csf */
If system_fix Then
   csf_zip_file = valid_csf || '.zip'
Else
   csf_zip_file = valid_csf
xrc = SysFileTree(csf_zip_file, 'csf_zip.', 'fo')
If xrc = 0 & csf_zip.0 <> 0 Then
   csf_zip = csf_zip_file
Else
   Do
      If Rx_fileexists(valid_fix || 'b.zip') Then
         csf_zip = '*'
   End
If \system_fix Then
   ecsapar_lst = csf_zip
/* look for fixes */
If csf_zip <> '' Then
   Do
      If system_fix Then
         csf_fix_file = valid_fix || '?.zip'
      Else
         Do
            If ecsapar_lst <> '' Then
               Do
                  If Abbrev(valid_fix, '\\') Then
                     Do
                        Parse Var valid_fix '\\' cd_msg '\' cd_label '\' cd_file
                        csf_fix_file = our_path || cd_file
                     End
                  Else
                     csf_fix_file = valid_fix
               End
         End
      xrc = SysFileTree(csf_fix_file, 'fix_zip.' , 'fo')
      If \ecs_pro Then
         valid_no = valid_no_uni
      Else
         valid_no = valid_no_smp
      If xrc = 0 & fix_zip.0 >= valid_no Then
         Do
            If system_fix Then
               Do
                  Call Rx_sayout 'Creating' valid_desc 'service files.'
                  /* create work directories */
                  csf_path = ecswork_dir || '\service\' || valid_fix
                  xrc = Rx_makepath(csf_path)
                  /* unzip service tool */
                  If csf_zip <> '*' Then
                     xrc = Rx_addresscmd('@unzip -oqq' csf_zip '-d' ecswork_dir || '\service', '-q')
                  srv_path = ecswork_dir || '\service\csf'
                  /* unzip fixes */
                  If csf_zip <> '*' Then
                     xrc = Rx_addresscmd('@unzip -oqq' csf_fix_file '-d' csf_path, '-q')
                  Else
                     xrc = Rx_addresscmd('@unzip -oqq' csf_fix_file '-d' ecswork_dir || '\service', '-q')
                  /* locate srv_prod.os2 and check */
                  srv_prod = csf_path || '\srv_prod.os2'
                  xrc = SysFileTree(srv_prod, 'srv_prod.', 'fso')
                  found_it = 0
                  If xrc = 0 & srv_prod.0 <> 0 Then
                     Do m = 1 To srv_prod.0
                        Call Stream srv_prod.m, 'c', 'close'
                        Call Stream srv_prod.m, 'c', 'open read'
                        fix_level = Charin(srv_prod.m, 147, 7)
                        Call Stream srv_prod.m, 'c', 'close'
                        If Wordpos(fix_level, valid_lvl) <> 0 Then
                           Do
                              found_it = 1
                              Call Stream srv_prod.m, 'c', 'close'
                              Call Stream srv_prod.m, 'c', 'open read'
                              fix_name = Translate(Charin(srv_prod.m, 139, 7), '0', '_')
                              Call Stream srv_prod.m, 'c', 'close'
                              Leave m
                           End
                     End
                  /* validate the fixpack */
                  If found_it Then
                     Do
                        srv_prod = csf_path || '\fix\os2.1\srv_prod.os2'
                        Call Stream srv_prod, 'c', 'close'
                        Call Stream srv_prod, 'c', 'open read'
                        fix_level = Charin(srv_prod, 147, 7)
                        Call Stream srv_prod, 'c', 'close'
                        If Wordpos(fix_level, valid_lvl) <> 0 Then
                           xrc = 1
                        Else
                           Do
                              Call Rx_sayout fix_level 'is not a valid fixpack for eCS.'
                              xrc = 0
                           End
                     End
                  Else
                     Do
                        Call Rx_sayout 'Invalid or damaged fixpack, aborting.'
                        xrc = 0
                     End
               End
            Else
               Do
                  ecs_fix_file = csf_fix_file
                  xrc = 1
               End
         End
      Else
         Do
            Call Rx_sayout 'Expected to find at least' valid_no 'zip file(s),'
            Call Rx_sayout 'but only found' fix_zip.0 'files, aborting.'
            service.0 = 0
         End
   End
Else
   Do
      Call Rx_sayout 'Service tool or ptf control file not found, aborting.'
      xrc = 0
   End

Return xrc

/*:VRX         Locate_target_drive
*/
Locate_target_drive:
/* locate potential target drive(s) for service */

my_drives. = ''
my_drives.0 = 0
add_our_drive = (Wordpos(SysFileSystemType(our_drive), 'CDFS LAN') = 0)
local_drives = SysDriveMap(, 'local')
Do i = 1 To Words(local_drives)
   vol_ltr = Word(local_drives, i)
   If add_our_drive & vol_ltr = our_drive Then
      add_our_drive = 0
   If SysDriveInfo(vol_ltr) = '' Then
      Iterate i
   If Wordpos(SysFileSystemType(vol_ltr), 'CDFS LAN') = 0 Then
      Do
         j = my_drives.0 + 1
         my_drives.j = vol_ltr
         my_drives.0 = j
      End
End
If add_our_drive Then
   Do
      j = my_drives.0 + 1
      my_drives.j = our_drive
      my_drives.0 = j
   End

/* make boot drive first */
Do i = 1 To my_drives.0
   If my_drives.i = boot_drive & i <> 1 Then
      Do
         save_drive = my_drives.1
         my_drives.1 = boot_drive
         my_drives.i = save_drive
         Leave i
      End
End

/* select a target drive */
locked_drive = ''
fixstart_drive = 'ERROR:'
fixstart_ecswork = 'ERROR:'
targets. = ''
targets.0 = 0
Do i = 1 To my_drives.0
   If SysFileSystemType(my_drives.i) <> 'HPFS' Then
      Iterate i
   config_sys = my_drives.i || '\config.sys'
   ecsreg = my_drives.i || '\os2\ecsreg.ini'
   sys_lvl = my_drives.i || '\OS2\INSTALL\SYSLEVEL.OS2'
   Parse Value Find_locklist(my_drives.i) With lock_exists lock_list lock_dir .
   If lock_exists & Filespec('n', lock_dir) = 'IBMCSFLK' Then
      locked_drive = my_drives.i
   If \ecs_pro Then
      ecs_pro = Rx_fileexists(my_drives.i || '\OS2\INSTALL\SMP')
   /* test for registered copy of eCS on the potential target drive(s) */
   If Rx_fileexists(config_sys) & Rx_fileexists(sys_lvl) ,
      & (SysIni(ecsreg, 'eComStationRegistration', 'SecureIt1') <> 'ERROR:') Then
      Do
         Call Stream sys_lvl, 'c', 'close'
         Call Stream sys_lvl, 'c', 'open read'
         sys_comp = Charin(sys_lvl, 141, 9)
         If (sys_comp = mcp_comp) | (sys_comp = acp_comp) Then
            Do
               sys_desc = Strip(Translate(Charin(sys_lvl, 61, 80), ' ', '00'x), 't')
               If ecs_pro Then
                  sys_desc = pro_desc
               Else
                  sys_desc = ecs_desc
               If fixstart_drive = 'ERROR:' Then
                  Do
                     fixstart_drive = SysIni(my_drives.i || '\ecs\ecsupdt.ini', 'FIXSTART', 'TARGET')
                     fixstart_service = SysIni(my_drives.i || '\ecs\ecsupdt.ini', 'FIXSTART', 'SERVICE')
                     fixstart_ecswork = SysIni(my_drives.i || '\ecs\ecsupdt.ini', 'FIXSTART', 'ECSWORK')
                  End
               Parse Value Get_kernel_info(my_drives.i) With . revision
               If Pos('W4', revision) = 0 Then
                  If Pos('SMP', revision) = 0 Then
                  Do
                     Call Rx_sayout 'The kernel on eCS drive' my_drives.i 'is not supported by eCS,'
                     Call Rx_sayout 'kernel revision =' revision || '.'
                     Call Rx_sayout ''
                     Iterate i
                  End
               Call Check_system my_drives.i
               j = targets.0 + 1
               targets.j = my_drives.i sys_desc
               targets.0 = j
            End
         Call Stream sys_lvl, 'c', 'close'
      End
End

/* test the validity of all zip files */
If \zips_checked Then
   Do
      Call Rx_sayout 'Checking integrity of zip files, please wait.'
      Call Rx_sayout ''
      frx = SysFileTree('*.zip', 'test.', 'fo')
      If frx = 0 Then
         Do i = 1 To test.0
            zip_name = Filespec('n', test.i)
            /* test the zip file */
            Address cmd 'zip -T' zip_name '2>&1|RXQUEUE'
            Do Queued()
               Parse Pull ans
               If Pos(Translate(zip_name), Translate(ans)) <> 0 Then
                  Do
                     If Pos('OK', ans) = 0 Then
                        Do
                           Call Rx_sayout ans
                           xrc = 0
                        End
                  End
            End
         End
   End

/* copy files from eCS Pro Installation CD or the smp base code */
If ecs_pro | (Rx_fileexists('ecspro10.zip') & Rx_fileexists('smp4501.zip')) Then
   Do
      pro_list. = ''
      x = 0
      /* special code for smp update */
      If Rx_fileexists('ecspro10.zip') & Rx_fileexists('smp4501.zip') Then
         Do
            If \Rx_fileexists(our_path || 'os2root') Then
               xrc = Rx_addresscmd('unzip -oqq ecspro10.zip -d' our_path || 'os2root')
         End
      Else
         Do
            x = x + 1; pro_list.x = '\os2\boot\os2apic.psd' our_path || 'os2root\os2\boot\os2apic.psd' 'os2apic.psd'
            x = x + 1; pro_list.x = '\os2\install\smp\*.*' our_path || 'os2root\os2\install\SMP\*.*' 'os2krnl'
         End
      x = x + 1; pro_list.x = '\ecs\os2root\os2krnl' our_path || 'os2root\os2krnl' 'os2krnl'
      x = x + 1; pro_list.x = '\ecs\os2root\os2ldr' our_path || 'os2root\os2ldr' 'os2ldr'
      x = x + 1; pro_list.x = '\os2\sesdd32.*' our_path || 'os2root\os2\*.*' 'sesdd32.sys'
      x = x + 1; pro_list.x = '\os2\dll\doscall1.dll' our_path || 'os2root\os2\dll\doscall1.dll' 'doscall1.dll'
      x = x + 1; pro_list.x = '\os2\pdpsi\pmdf\*.*' our_path || 'os2root\os2\pdpsi\pmdf\*.*' 'pmdf.exe'
      x = x + 1; pro_list.x = '\os2\system\trace\*.*' our_path || 'os2root\os2\system\trace\*.*' 'os2krnl.sym'
      pro_list.0 = x
      Do x = 1 To pro_list.0 While ecs_pro
         Parse Var pro_list.x pro_source pro_target pro_file
         If \Rx_fileexists(Filespec('d', pro_target) || Filespec('p', pro_target) || pro_file) Then
            Do
               install_cd = Insert_cd('eCS_CDRun', 'Please insert the eCS Installation CD')
               If install_cd <> '' Then
                  Do
                     pro_dir = Filespec('d', pro_target) || Strip(Filespec('p', pro_target), 't', '\')
                     Call Rx_makepath pro_dir
                     frc = Rx_addresscmd('@copy' install_cd || pro_source pro_target, '-q')
                  End
               Else
                  ecs_pro = 0
            End
      End
   End

/* copy fixes from eCS CD(s) */
Do i = 1 To valid.0
   Parse Var valid.i . . '|' valid_fix '|' .
   valid_fix = Strip(valid_fix)
   If Abbrev(valid_fix, '\\') Then
      Do
         Parse Var valid_fix '\\' cd_msg '\' cd_label '\' cd_file
         If \Rx_fileexists(our_path || cd_file) Then
            Do
               cdwork_dir = our_path || Strip(Filespec('p', 'z:\' || cd_file), , '\')
               Call Rx_makepath cdwork_dir
               cd_letter = Insert_cd(cd_label, 'Please insert' cd_msg)
               If cd_letter <> '' Then
                  frc = Rx_addresscmd('@copy' cd_letter || '\' || cd_file cdwork_dir, '-q')
            End
      End
End

/* select a target drive for service */
If targets.0 <> 0 Then
   Do
      If targets.0 <> 1 Then
         Do
            Call Rx_sayout 'Please select the drive for processing.', , -1
            Do i = 1 To targets.0
               Call Rx_sayout i '- Drive' Word(targets.i, 1) Subword(targets.i, 2) || '.', , -1
            End
            Call Rx_sayout 'Any invalid selection will exit.', 4, -1
            Parse Upper Pull ans .
         End
      Else
         ans = 1
      If ans >= 1 & ans <= targets.0 Then
         Do
            target_drive = Word(targets.ans, 1)
            /* test for interrupted service */
            If fixstart_drive <> 'ERROR:' Then
               Do
                  If fixstart_ecswork <> 'ERROR:' Then
                     Do
                        Call Rx_sayout 'Drive' fixstart_drive 'has interrupted service pending,'
                        Call Rx_sayout 'and you can attempt to complete the interrupted service.', 1
                        Call Rx_sayout 'When you hear a "beep" after the IBM service processing has started,'
                        Call Rx_sayout 'open the "fservice.exe" window and follow the instructions.', 1
                        Call Rx_sayout 'Press "enter" to attempt the recovery or enter any other response to exit.'
                        Parse Upper Pull ans .
                        If ans <> '' Then
                           Return 0
                        Else
                           target_drive = fixstart_drive
                     End
                  Else
                     Do
                        Call Rx_sayout 'Drive' fixstart_drive 'has interrupted service that'
                        Call Rx_sayout 'cannot be recovered automatically.'
                        Return 0
                     End
               End
            target_os2ini = target_drive || '\OS2\OS2.INI'
            config_sys = target_drive || '\config.sys'
            startup_cmd = Stream(target_drive || '\startup.cmd', 'c', 'query exists')
            sys_lvl = target_drive || '\os2\install\syslevel.os2'
            Call Stream sys_lvl, 'c', 'close'
            Call Stream sys_lvl, 'c', 'open read'
            sys_curr = Charin(sys_lvl, 45, 7)
            sys_suffix = Translate(Substr(sys_curr, 4, 4)) || Left(target_drive, 1)
            sys_base = Charin(sys_lvl, 53, 7)
            Call Stream sys_lvl, 'c', 'close'
            /* setup ecs directory */
            ecsbase_dir = target_drive || '\ECS'
            Call Rx_makepath ecsbase_dir
            ecsupdt_ini = target_drive || '\ecs\ecsupdt.ini'
            /* define log file */
            log_file = target_drive || '\ECS\ECSUPDT.LOG'
            Call Stream log_file, 'c', 'close'
            Call Stream log_file, 'c', 'open write'
            log_out = 1
            Call Rx_sayout '==========', , 1
            Call Rx_sayout 'Starting' our_prog version_info, , 1
            Do x = 1 To log_lines.0
               Parse Var log_lines.x p1 p2 msg
               Call Rx_sayout msg, p1, p2
            End
            log_lines.0 = 0
            /* setup backup directories */
            ecsback_dir = target_drive || '\ECS\ARCHIVE\BASE\ECS' || sys_suffix
            Call Rx_makepath ecsback_dir
            ptfback_dir = target_drive || '\ECS\ARCHIVE\PTFBACK'
            Call Rx_makepath ptfback_dir
            /* copy any old fix backups to the correct directory */
            frc = SysFileTree(target_drive || '\ECS\ARCHIVE\*.bak', 'old.', 'fso')
            If frc = 0 Then
               Do x = 1 To old.0
                  If Filespec('p', old.x) <> '\ECS\ARCHIVE\PTFBACK\' Then
                     frc = Rx_addresscmd('move' old.x Substr(ptfback_dir, 3), '-q')
               End
            /* check for valid syslevel.os2 and userdirs.os2 */
            Call Check_syslevel(sys_lvl)
            service_log = target_drive || '\OS2\INSTALL\SERVICE.LOG'
            dlg_inst = Rx_fileexists(target_drive || '\ecs\install\dlg_inst.dat')
            If dlg_inst Then
               Do
                  If \Rx_fileexists(target_drive || '\ECS\INSTALL') Then
                     Call Rx_makepath target_drive || '\ECS\INSTALL'
               End
            dlgback_dir = target_drive || '\ECS\DLG\BACKUP'
            theme_dir = target_drive || '\os2\thememgr'
            theme_zip = theme_dir || '\thm' || Substr(sys_curr, 4) || '.zip'
            theme_upd = Rx_fileexists(theme_dir)
            theme_idx = theme_dir || '\thememgr.idx'
            theme_bmp = theme_dir || '\thmbmp.zip'
            theme_res = theme_dir || '\thmres.zip'
            Parse Value Find_locklist(target_drive) With lock_exists lock_list lock_dir .
            If locked_drive <> '' ,
               & locked_drive <> target_drive ,
               & target_drive <> boot_drive Then
               Do
                  Call Rx_sayout 'Drive' locked_drive 'has service that has not completed.'
                  Call Rx_sayout 'Please boot to drive' locked_drive 'before attempting any', 1
                  Call Rx_sayout 'service processing on a drive other than' locked_drive || '.', 1
                  Return 0
               End
            /* test for smp support */
            Parse Value Get_kernel_info(target_drive) With smp_kernel revision
            os2apic = target_drive || '\os2\boot\os2apic.psd'
            If smp_kernel Then
               kernel_desc = 'IBM SMP Kernel'
            Else
               kernel_desc = 'IBM Kernel'
            Call Rx_sayout 'Found' kernel_desc '- Revision:' revision 'on drive' target_drive || '.'
            If smp_kernel Then
               upd_desc = pro_desc
            Else
               upd_desc = ecs_desc
            If Rx_fileexists('ecspro10.zip') ,
               & \Rx_fileexists(target_drive || '\os2\install\smp') Then
               xrc = Rx_addresscmd('unzip -oq ecspro10.zip -d' target_drive)
            /* test for ecs pro */
            ecs_pro = Rx_fileexists(target_drive || '\OS2\INSTALL\SMP')
            If ecs_pro Then
               upd_desc = pro_desc
            Else
               upd_desc = ecs_desc
            /* check for userdirs.os2 */
            userdirs = target_drive || '\os2\install\userdirs.os2'
            If Rx_readlines(userdirs) Then
               Do
                  valid_userdirs = 0
                  Do x = 1 To file_lines.0
                     If Space(file_lines.x) = '' Then
                        Iterate x
                     If Rx_fileexists(Space(file_lines.x)) Then
                        valid_userdirs = 1
                  End
                  /* no valid entries in userdirs.os2 */
                  If \valid_userdirs Then
                     Call SysFileDelete userdirs
                  file_lines.0 = 0
               End
            /* check for previous maintenance */
            logstart = Filespec('d', sys_lvl) || Filespec('p', sys_lvl) || 'logf0000.os2'
            If Rx_fileexists(logstart) Then
               Do
                  Call Stream logstart, 'c', 'close'
                  Call Stream logstart, 'c', 'open read'
                  archive_path = Strip(Charin(logstart, 424, 256), 't', '00'x)
                  Call Stream logstart, 'c', 'close'
                  If Rx_fileexists(archive_path) Then
                     archived = 1
                  Else
                     Do
                        arch_lvl = sys_base
                        Call Rx_sayout 'Archive path' archive_path 'not found.'
                        archived = 0
                        Return 0
                     End
               End
            Else
               Do
                  archive_path = target_drive || '\ECS\ARCHIVE\ECS' || sys_suffix
                  Call Rx_makepath archive_path
                  archived = 0
               End
            backup_path = target_drive || '\ECS\ARCHIVE\BACKUP\ECS' || sys_suffix
            Call Rx_makepath target_drive || backup_path
            If Select_fixpack() Then
               Do
                  sys_lvl = target_drive || valid_sys
                  Call Stream sys_lvl, 'c', 'close'
                  Call Stream sys_lvl, 'c', 'open read'
                  sys_curr = Charin(sys_lvl, 45, 7)
                  sys_base = Charin(sys_lvl, 53, 7)
                  sys_desc = Strip(Translate(Charin(sys_lvl, 61, 80), ' ', '00'x), 't')
                  sys_comp = Charin(sys_lvl, 141, 9)
                  Call Stream sys_lvl, 'c', 'close'
                  Call Rx_sayout valid_desc 'will be used for processing on drive' target_drive || '.', , 1
               End
            Else
               Return 0
         End
      Else
         Do
            Call Rx_sayout 'No target drive selected, exiting.'
            Return 0
         End
   End
Else
   Do
      Call Rx_sayout 'This update is only for registered copies of eComStation'
      Call Rx_sayout 'component id' mcp_comp || '.', 1
      Return 0
   End

/* test for space on target drive */
target_free = 0
If system_fix Then
   Do
      If archived Then
         target_free = target_free + 4                      /* backup */
      Else
         Do
            target_free = target_free + 16                 /* archive */
            If Rx_fileexists(target_drive || '\MMOS2') Then
               target_free = target_free + 2
         End
      If \Rx_fileexists(ecsback_dir || '\npbase.zip') Then
         target_free = target_free + 1
      If dlg_inst & \Rx_fileexists(dlgback_dir || '\dlgc001.zip') Then
         target_free = target_free + 4
      If theme_upd & \Rx_fileexists(theme_dir || '\thmc001.zip') Then
         target_free = target_free + 1
   End
Else
   target_free = valid_free
free = Rx_driveinfo(target_drive, '-fr')
If free < target_free Then
   Do
      Call Rx_sayout 'Service processing could require' target_free 'Mb of free space on target drive' target_drive || ','
      Call Rx_sayout 'but drive' target_drive 'does not have sufficient space (' || free || ' Mb) for service processing.', 1
      If archived Then
         Do
            Call Rx_sayout 'An archive already exists on target drive' target_drive || ','
            Call Rx_sayout 'please free more than' (target_free - free) 'Mb on drive' target_drive || '.', 1
            Return 0
         End
      Else
         Do
            Call Rx_sayout 'Please select a drive for the archive.'
            temp. = ''
            temp.0 = 0
            all_drives = SysDriveMap(, 'local')
            Do i = 1 To Words(all_drives)
               vol_ltr = Word(all_drives, i)
               free = Rx_driveinfo(vol_ltr, '-fr')
               If free > target_free Then
                  Do
                     j = temp.0 + 1
                     temp.j = drive '-' free 'Mb free'
                     temp.0 = j
                  End
            End
            If temp.0 <> 0 Then
               Do
                  Do i = 1 To temp.0
                     Call Rx_sayout i '- Drive' temp.i
                  End
                  Call Rx_sayout 'An invalid selection will exit'
                  Parse Pull ans
                  If ans >= 1 & ans <= temp.0 Then
                     Do
                        new_archive = Word(temp.ans, 1)
                        Call Rx_sayout 'Drive' new_archive 'has been selected for archiving'
                        ecsback_dir = new_archive || '\ECS\ARCHIVE\BASE\ECS' || sys_suffix
                        Call Rx_makepath ecsback_dir
                        archive_path = new_archive || '\ECS\ARCHIVE\ECS' || sys_suffix
                        backup_path = new_archive || '\ECS\ARCHIVE\BACKUP\ECS' || sys_suffix
                     End
                  Else
                     Do
                        Call Rx_sayout 'No drive with sufficient space for the archive was selected, exiting'
                        Return 0
                     End
               End
            Else
               Do
                  Call Rx_sayout 'No drive with sufficient space was found for the archive, exiting'
                  Return 0
               End
         End
   End

/* locate a work drive */
work_drive = ''
work_drive_free = 0
If fixstart_ecswork <> 'ERROR:' Then
   Do
      work_drive = Filespec('d', fixstart_ecswork)
      work_drive_free = Rx_driveinfo(work_drive, '-fr')
   End
If work_drive = '' Then
   Do
      local_drives = SysDriveMap(, 'local')
      Do i = 1 To Words(local_drives)
         vol_ltr = Word(local_drives, i)
         If SysDriveInfo(vol_ltr) = '' Then
            Iterate i
         If Wordpos(SysFileSystemType(vol_ltr), 'HPFS JFS') <> 0 Then
            Do
               free = Rx_driveinfo(vol_ltr, '-fr')
               If (vol_ltr = Filespec('d', archive_path)) Then
                  free = free - target_free
               If free > work_drive_free Then
                  Do
                     work_drive_free = free
                     work_drive = vol_ltr
                  End
            End
      End
   End
/* check for work space */
If work_drive_free < work_free Then
   Do
      Call Rx_sayout 'Service processing could require' work_free 'Mb of free space on work drive' work_drive
      Call Rx_sayout 'Work drive' work_drive 'does not have sufficient space (' || work_drive_free || ' Mb)'
      Call Rx_sayout 'for service processing', 1
      Return 0
   End

backup_name = ptfback_dir || '\BACKOUT.BAK'
action_list. = ''
x = 0
If system_fix Then
   Do
      If \smp_kernel Then
         Do
            If Translate(sys_curr) < Translate(valid_fix, '0', '_') Then
               Do
                  x = x + 1; action_list.x = 'Apply' valid_desc
               End
            If Translate(sys_curr) = Translate(valid_fix, '0', '_') Then
               Do
                  x = x + 1; action_list.x = 'Re-apply' valid_desc
                  x = x + 1; action_list.x = 'Skip application of' valid_fix 'and process the Serenity Systems updates'
               End
            If archived & (Translate(sys_curr) > Translate(sys_base)) Then
               Do
                  x = x + 1; action_list.x = 'Backout' valid_desc
               End
         End
      If ecs_pro & \lock_exists Then
         Do
            If \smp_kernel & system_fix Then
               Do
                  If Translate(sys_curr) < Translate(valid_fix, '0', '_') Then
                     Do
                        x = x + 1; action_list.x = 'Install SMP support (for SMP at the' valid_fix 'level, apply' valid_fix 'first)'
                     End
                  Else
                     Do
                        x = x + 1; action_list.x = 'Install SMP support at the' valid_fix 'level'
                     End
               End
            If smp_kernel & system_fix Then
               Do
                  x = x + 1; action_list.x = 'Uninstall SMP support'
               End
         End
      /*
      Call Rx_sayout 'C<ommit> applied IBM service', 1
      */
      If fixstart_drive <> 'ERROR:' Then
         Do
            action_list. = ''
            x = 0
            If fixstart_service Then
               Do
                  x = x + 1; action_list.x = 'Apply' valid_desc '(recover)'
               End
            Else
               Do
                  x = x + 1; action_list.x = 'Backout' valid_desc '(recover)'
               End
         End
   End
Else
   Do
      If \Abbrev(Translate(valid_csf), 'ECSPRO') Then
         Do
            Parse Var valid_csf backout_csf_name '.' .
            backout_csf = Strip(backout_csf_name) || '.bol'
            backup_name = ptfback_dir || '\' || backout_csf_name || '.BAK'
            If \Rx_fileexists(backup_name) Then
               Do
                  x = x + 1; action_list.x = 'Apply' valid_desc
               End
            Else
               Do
                  x = x + 1; action_list.x = 'Reapply' valid_desc
               End
            If Rx_fileexists(backout_csf) & Rx_fileexists(backup_name) Then
               Do
                  x = x + 1; action_list.x = 'Backout' valid_desc
               End
         End
      Else
         Do
            If \smp_kernel Then
               Do
                  x = x + 1; action_list.x = 'Install SMP support'
               End
            Else
               Do
                  x = x + 1; action_list.x = 'Uninstall SMP support'
               End
         End
   End
action_list.0 = x
If action_list.0 <> 1 | system_fix | Abbrev(Translate(valid_csf), 'ECSPRO') Then
   Do
      If fixstart_drive = 'ERROR:' Then
         Do
            Call Rx_sayout 'Please select a service -', ,-1
            Do x = 1 To action_list.0
               Call Rx_sayout x '-' action_list.x, , -1
            End
            Call Rx_sayout 'Any invalid selection will exit', 4, -1
            Parse Upper Pull ans .
         End
      Else
         ans = 1
   End
Else
   ans = 1
If ans >= 1 & ans <= action_list.0 Then
   ans = action_list.ans
Else
   ans = 'Exit'
Select
   When Abbrev(ans, 'A') | Abbrev(ans, 'R') Then
      Do
         apply = 1
         action_desc = 'apply'
      End
   When Abbrev(ans, 'S') Then
      Do
         skip = 1
         action_desc = 'bypass'
      End
   When Abbrev(ans, 'B') Then
      Do
         action_desc = 'backout'
         backout = 1
         If \system_fix Then
            Do
               valid_csf = backout_csf
               valid_fix = backup_name
            End
      End
   When Abbrev(ans, 'I') Then
      Do
         action_desc = 'SMP install'
         install_smp = 1
      End
   When Abbrev(ans, 'U') Then
      Do
         action_desc = 'SMP uninstall'
         uninstall_smp = 1
      End
   /*
      When Abbrev(ans, 'C') Then
         Do
            action_desc = 'commit'
            commit = 1
         End
   */
   Otherwise
      Return 0
End

Select
   When install_smp & smp_kernel Then
      Do
         Call Rx_sayout 'You have elected to do' action_desc 'processing'
         Call Rx_sayout 'on a system that is already at that level', 1
         Return 0
      End
   When uninstall_smp & \smp_kernel Then
      Do
         Call Rx_sayout 'You have elected to do' action_desc 'processing'
         Call Rx_sayout 'on a system that is not at that level', 1
         Return 0
      End
   When apply & smp_kernel & (sys_curr = sys_base) Then
      Do
         Call Rx_sayout 'You have elected to do' action_desc 'processing'
         Call Rx_sayout 'Please remove SMP support before applying service', 1
         Return 0
      End
   When install_smp & (sys_curr <> sys_base) Then
      Do
         Call Rx_sayout 'You have elected to do' action_desc 'processing'
         Call Rx_sayout 'The SMP update will be applied with service processing', 1
         apply = 1
         smp_upgrade = 1
      End
   When uninstall_smp & (sys_curr <> sys_base) Then
      Do
         Call Rx_sayout 'You have elected to do' action_desc 'processing'
         Call Rx_sayout 'The SMP update will be removed with eCS processing', 1
         smp_upgrade = 1
      End
   When install_smp & (sys_curr = sys_base) Then
      Do
         Call Rx_sayout 'You have elected to do' action_desc 'processing'
         Call Rx_sayout 'The SMP update will be applied with eCS processing', 1
         smp_upgrade = 1
      End
   When uninstall_smp & (sys_curr = sys_base) Then
      Do
         Call Rx_sayout 'You have elected to do' action_desc 'processing'
         Call Rx_sayout 'The SMP update will be removed with eCS processing', 1
         smp_upgrade = 1
      End
   Otherwise
      Nop
End

If upd_desc <> sys_desc Then
   xrc = Update_syslevel(, upd_desc)
Else
   xrc = 1
Call Stream sys_lvl, 'c', 'close'
Call Stream sys_lvl, 'c', 'open read'
sys_desc = Strip(Translate(Charin(sys_lvl, 61, 80), ' ', '00'x), 't')
Call Stream sys_lvl, 'c', 'close'

If xrc & \skip Then
   Do
      Call Rx_sayout 'Preparing your system for' action_desc 'processing on'
      Call Rx_sayout sys_desc 'at level' sys_curr 'on drive' target_drive, 1
   End

Return xrc

/*:VRX         Misc_updates
*/
Misc_updates:
/* make some work areas and do misc stuff */

If fixstart_ecswork = 'ERROR:' Then
   ecswork_dir = SysTempFileName(work_drive || '\ECS????')
Else
   ecswork_dir = fixstart_ecswork
If Rx_makepath(ecswork_dir) Then
   Call Rx_sayout 'Created work directory' ecswork_dir
Else
   Do
      Call Rx_sayout 'Unable to create work directory' ecswork_dir
      Return 0
   End

/* back up original dialog enhancer modules if necessary */
If dlg_inst & (sys_curr = sys_base) & Rx_fileexists(dlgback_dir) Then
   Do
      dlg_back = dlgback_dir || '\dlg' || Substr(sys_curr, 4) || '.zip'
      dlg_data = dlgback_dir || '\*.*'
      If \Rx_fileexists(dlg_back) Then
         Call Rx_addresscmd '@zip -qjom' dlg_back dlg_data
   End

/* back up original net installer modules if necessary */
If apply & \Rx_fileexists(ecsback_dir || '\npbase.zip') Then
   Do
      Call Rx_addresscmd '@zip -qjoDS' ecsback_dir || '\npbase.zip' target_drive || '\ibminst\npconfig.exe'
      Call Rx_addresscmd '@zip -qjoDS' ecsback_dir || '\npbase.zip' target_drive || '\ibminst\npresdll.dll'
      Call Rx_addresscmd '@attrib +r' ecsback_dir || '\npbase.zip', '-q'
   End

/* check theme manager */
If theme_upd Then
   Do
      theme_default = 'eComStation - classic theme' || '00'x
      If SysIni(target_os2ini, 'ThemeManager', 'CurrentTheme') = 'ERROR:' Then
         Call SysIni target_os2ini, 'ThemeManager', 'CurrentTheme', theme_default
      theme_curr = Strip(SysIni(target_os2ini, 'ThemeManager', 'CurrentTheme'), 't', '00'x)
      /* delete pmmrgres.dll so that it will not get serviced */
      If Rx_fileexists(theme_dir || '\pmmrgres.dll') Then
         Call SysFileDelete theme_dir || '\pmmrgres.dll'
      /* back up original theme manager updates if necessary */
      If (sys_curr = sys_base) & \Rx_fileexists(theme_zip) Then
         Call Rx_addresscmd '@zip -joq' theme_zip theme_dir || '\*.dll'
      If \Rx_fileexists(theme_bmp) Then
         Call Rx_addresscmd '@zip -joqm' theme_bmp theme_dir || '\*.bmp'
      If \Rx_fileexists(theme_res) Then
         Call Rx_addresscmd '@zip -joqm' theme_res theme_dir || '\*.res'
   End

Return 1

/*:VRX         Prepare_ecs_backup_target
*/
Prepare_ecs_backup_target:

If Rx_addresscmd('@attrib -r -s -h' target_drive || '\os2\system\trace\*', '-q') Then
   If Rx_addresscmd('@erase' target_drive || '\os2\system\trace\* /N', '-q') Then
   xrc = 1
Else
   xrc = 0
If xrc Then
   Do
      If Rx_addresscmd('@attrib -r -s -h' target_drive || '\os2\pdpsi\pmdf\*', '-q') Then
         If Rx_addresscmd('@erase' target_drive || '\os2\pdpsi\pmdf\* /n', '-q') Then
         xrc = 1
   End
Else
   xrc = 0
If xrc Then
   Do
      If Rx_addresscmd('@attrib -r -h -s' target_drive || '\os2krnl', '-q') Then
         If Rx_addresscmd('@attrib -r -h -s' target_drive || '\os2ldr', '-q') Then
         If Rx_addresscmd('@attrib -r -h -s' target_drive || '\os2\sesdd32.sys', '-q') Then
         If Rx_addresscmd('@attrib -r -h -s' target_drive || '\os2\dll\doscall1.dll', '-q') Then
         xrc = 1
   End
Else
   xrc = 0
If \xrc Then
   Call Rx_sayout 'Unable to prepare the system for uninstall'
Return xrc

/*:VRX         Read_patches
*/
Read_patches: Procedure Expose (log_parms) system_fix ecsback_dir system_updates ,
   config_sys our_path ecswork_dir target_drive fix_name patch_list. replace_newer ,
   lock_exists lock_list archive_path backup_path backup_name apply backout file_lines.
save_trace = Trace('N')
Parse Arg patch_list

Parse Value Translate(Filespec('n', patch_list)) With patch_list_name '.' .

system_update = replace_newer & Wordpos(patch_list_name, system_updates) <> 0
java_list = Abbrev(patch_list_name, 'JRT')


backup_now = \Rx_fileexists(backup_name) & \system_fix & apply

patch_list. = ''
patch_list.0 = 0

/* read in the patch list */
Call Rx_sayout 'Reading eCS patch list file -' patch_list, , 1
If backup_now & \java_list Then
   Call Rx_sayout 'Any files referenced in' patch_list 'will be zipped to' backup_name, 1
If Rx_readlines(patch_list) Then
   Do i = 1 To file_lines.0
      file_lines.i = Space(file_lines.i)
      If Abbrev(file_lines.i, ';') ,
         | file_lines.i = '' ,
         | Abbrev(file_lines.i, '1a'x) Then
         Iterate i
      /* handle command */
      If Abbrev(Translate(file_lines.i), 'ADDRESS CMD') Then
         Do
            command_line = 'CMD'
            input_line = Subword(file_lines.i, 3)
         End
      Else
         Do
            command_line = Translate(Word(file_lines.i, 1))
            If Abbrev(command_line, 'MOVE') Then
               command_line = 'COPY'
            input_line = Subword(file_lines.i, 2)
         End
      /* adjust drive pointers */
      Do j = 1 To Words(input_line)
         input_word = Word(input_line, j)
         Select
            /* points to current work directory */
            When Abbrev(input_word, '..\') Then
               input_word = ecswork_dir || Substr(input_word, 3)
            /* points to current boot drive */
            When Abbrev(input_word, '.\') Then
               input_word = target_drive || Substr(input_word, 2)
            /* points to current CD drive */
            When Abbrev(input_word, '\\') Then
               Do
                  cd_path = Subword(input_line, j)
                  Parse Var cd_path '\\' cd_msg '\' cd_label '\' cd_file .
                  cd_path = Strip('\\' || cd_msg || '\' || cd_label || '\' || cd_file)
                  j = j + Words(cd_path) - 1
                  input_word = our_path || cd_file
                  If \Rx_fileexists(input_word) Then
                     Do
                        Call Rx_sayout 'Source for' input_word 'not found', 1, 1
                        Iterate i
                     End
               End
            /* points to our current directory */
            When Abbrev(input_word, '\') Then
               input_word = '"' || our_path || Substr(input_word, 2) || '"'
            /* points to key file in path of target drive */
            When Abbrev(input_word, '$') Then
               Do
                  Parse Var input_word '$' key_file '|' key_path
                  key_file = Strip(key_file)
                  key_dir = Strip(Filespec('p', 'x:' || key_file), 't', '\')
                  key_file = Filespec('n', 'x:' || key_file)
                  key_path = Strip(key_path)
                  /* search path */
                  Parse Value Rx_findlines('set path=', config_sys) With ,
                     path_line_no path_line '|' .
                  If path_line_no <> 0 Then
                     Do
                        Parse Var path_line . '=' path_data
                        If key_dir <> '' Then
                           Do
                              key_pos = Pos(Translate(key_dir), Translate(path_data))
                              If key_pos <> 0 Then
                                 path_data = Substr(path_data, key_pos - 2)
                           End
                        Call Value 'ecsapar_path', path_data, 'os2environment'
                        test_path = SysSearchPath('ecsapar_path', key_file)
                        Call Value 'ecsapar_path', '', 'os2environment'
                        If key_path <> '' Then
                           input_word = Strip(Filespec('d', test_path) || key_path)
                        Else
                           Do
                              Call Rx_sayout 'Target drive for' key_file 'not found', 1, 1
                              Iterate i
                           End
                     End
                  Else
                     Do
                        Call Rx_sayout 'Target drive for' key_file 'not found', 1, 1
                        Iterate i
                     End
               End
            /* points to key file (@xxxxxx) in directory */
            When Abbrev(input_word, '@') Then
               Do
                  Parse Var input_word key_file '\' test_name
                  test = SysFileTree(target_drive || '\' || key_file, 'test.', 'fso')
                  If test = 0 & test.0 <> 0 Then
                     Do
                        input_word = Filespec('d', test.1) || Filespec('p', test.1) || test_name
                     End
                  Else
                     Do
                        Call Rx_sayout 'Target directory for' input_word 'not found', 1, 1
                        Iterate i
                     End
               End
            Otherwise Nop
         End
         command_line = command_line input_word
      End
      command_line = Space(command_line)
      Select
         When Abbrev(command_line, 'CMD') ,
            | Abbrev(command_line, 'BACKUP') ,
            | Abbrev(command_line, 'UNPACK') ,
            | Abbrev(command_line, 'UNZIP') Then
            Do
               Select
                  When Abbrev(command_line, 'BACKUP') Then
                     Do
                        If backup_now Then
                           Do
                              Parse Var command_line . source_file
                              Call Rx_sayout 'Backing up' source_file 'to' backup_name
                              Address cmd 'zip -rqoDS' backup_name source_file '2>&1|RXQUEUE'
                              Do Queued()
                                 Parse Pull ans
                                 Call Rx_sayout ans, , 1
                              End
                           End
                     End
                  When Abbrev(command_line, 'UNZIP') ,
                     | Abbrev(command_line, 'UNPACK') Then
                     Do
                        If Abbrev(command_line, 'UNZIP') Then
                           Do
                              Parse Var command_line . opts source_file 'to:' target
                              If target = '' Then
                                 Parse Var command_line . opts source_file target
                           End
                        Else
                           Parse Var command_line . source_file 'to:' target
                        opts = Strip(opts)
                        source_file = Strip(source_file)
                        target = Strip(target)
                        Interpret 'source_file =' source_file
                        If Rx_fileexists(source_file) Then
                           Do
                              Interpret 'target =' target
                              If Rx_fileexists(target) Then
                                 Do
                                    If Abbrev(command_line, 'UNZIP') Then
                                       Address cmd 'unzip' opts source_file '-d' target '2>&1|RXQUEUE'
                                    Else
                                       Address cmd 'unpack' source_file target '2>&1|RXQUEUE'
                                    Do Queued()
                                       Parse Pull ans
                                       Call Rx_sayout ans, , 1
                                    End
                                 End
                              Else
                                 Call Rx_sayout Word(command_line, 1) 'target directory' target 'not found'
                           End
                        Else
                           Call Rx_sayout Word(command_line, 1) 'source' source_file 'not found', , 1
                     End
                  Otherwise
                     Do
                        Interpret 'ADDRESS' command_line
                        Do Queued()
                           Parse Pull ans
                           Call Rx_sayout ans, , 1
                        End
                     End
               End
            End
         When Abbrev(command_line, 'COPY') Then
            Do
               Parse Var command_line c_first source_file target c_last
               If Rx_fileexists(source_file) | backout Then
                  Do
                     If Filespec('n', target) = '*' Then
                        Do
                           target = Filespec('d', target) || Strip(Filespec('p', target)) || Filespec('n', source_file)
                           command_line = c_first source_file target c_last
                        End
                     If Rx_fileexists(Filespec('d', target) || Strip(Filespec('p', target), 't', '\')) Then
                        Do
                           test_target = target
                           If Rx_fileexists(test_target) & apply Then
                              Do
                                 test_target = Check_lock_list(target)
                                 test_comp = Rx_compare(source_file, test_target)
                                 test_backdate = SysGetFileDateTime(source_file, 'w') <= SysGetFileDateTime(test_target, 'w')
                                 test_newdate = SysGetFileDateTime(source_file, 'w') > SysGetFileDateTime(test_target, 'w')
                              End
                           Else
                              Parse Value '0 0 1' With test_comp test_backdate test_newdate
                           If test_backdate & system_update Then
                              test_backdate = 0
                           If test_backdate & Translate(Filespec('n', target)) = 'OS2LDR' Then
                              test_backdate = 0
                           If Rx_fileexists(test_target) & backup_now Then
                              Call Rx_addresscmd 'zip -oqqSDX' backup_name test_target
                           If \test_comp | test_newdate | backout Then
                              Do
                                 If test_backdate & \backout Then
                                    Do
                                       Call Rx_sayout 'Update of' target 'would be a date regression', , 1
                                       Iterate i
                                    End
                                 If Rx_fileexists(source_file) Then
                                    Do
                                       x = patch_list.0 + 1
                                       patch_list.x = command_line
                                       patch_list.0 = x
                                    End
                                 Else
                                    Do
                                       If backout Then
                                          Do
                                             x = patch_list.0 + 1
                                             patch_list.x = 'ERASE' target
                                             patch_list.0 = x
                                          End
                                       Else
                                          Call Rx_sayout 'Source for' source_file 'does not exist', , 1
                                    End
                              End
                           Else
                              Call Rx_sayout 'Equal files, update of' target 'bypassed', , 1
                        End
                     Else
                        Call Rx_sayout 'Target directory for' target 'does not exist', , 1
                  End
               Else
                  Call Rx_sayout 'Source for' source_file 'does not exist', , 1
            End
         When Abbrev(command_line, 'PATCH') Then
            Do
               Parse Var command_line . pat_file target .
               If \Rx_fileexists(pat_file) Then
                  Do
                     Call Rx_sayout 'Patch file' pat_file 'not found', , 1
                     Iterate i
                  End
               /* locate target file */
               target = Check_lock_list(target)
               If \Rx_fileexists(target) Then
                  Do
                     Call Rx_sayout 'Target file' target 'for' pat_file 'not found', , 1
                     Iterate i
                  End
               /* update patch file with target file */
               frc = Rx_readlines(pat_file)
               If frc Then
                  Do
                     If Abbrev(file_lines.1, 'FILE') Then
                        Do
                           file_lines.1 = 'FILE' target
                           pat_file = ecswork_dir || '\' || pat_file
                           frc = Rx_writelines(pat_file)
                           If frc Then
                              Do
                                 x = patch_list.0 + 1
                                 patch_list.x = 'PATCH' pat_file '/A'
                                 patch_list.0 = x
                              End
                           Else
                              Do
                                 Call Rx_sayout 'Error updating patch file' pat_file, , 1
                                 Iterate i
                              End
                        End
                     Else
                        Do
                           Call Rx_sayout 'Invalid patch file format for' pat_file, , 1
                           Iterate i
                        End
                  End
            End
         When Abbrev(command_line, 'ERASE') Then
            Do
               x = patch_list.0 + 1
               patch_list.x = command_line
               patch_list.0 = x
            End
         When Abbrev(command_line, 'ATTRIB') Then
            Do
               x = patch_list.0 + 1
               patch_list.x = command_line
               patch_list.0 = x
            End
         When Abbrev(command_line, 'CALL') Then
            Do
               x = patch_list.0 + 1
               patch_list.x = command_line
               patch_list.0 = x
            End
         When Abbrev(command_line, 'NOTE') ,
            | Abbrev(command_line, 'PAUSE') Then
            Do
               x = patch_list.0 + 1
               patch_list.x = command_line
               patch_list.0 = x
            End
         When Abbrev(command_line, 'MSG') Then
            Do
               Call Rx_sayout Subword(command_line, 2)
            End
         Otherwise Do
            Call Rx_sayout 'Unknown command in' patch_list
            Call Rx_sayout command_line, 1
         End
      End
   End
file_lines.0 = 0
Trace(save_trace)
Return 1

/*:VRX         Restore_ecs_backup
*/
Restore_ecs_backup:

If Rx_addresscmd('@unzip -joqq' ecsback_dir || '\tracemcp.zip -d' target_drive || '\os2\system\trace') Then
   If Rx_addresscmd('@unzip -joqq' ecsback_dir || '\pmdfmcp.zip -d' target_drive || '\os2\pdpsi\pmdf') Then
   Do
      xrc = 1
      /*
            If Rx_fileexists(target_drive || '\ibmlan\netprog') Then
               xrc = Rx_repmod(target_drive || '\ibmlan\netprog\netwksta.ecs', ,
               target_drive || '\ibmlan\netprog\NETWKSTA.200', '-fq')
      */
   End
Else
   xrc = 0
If xrc Then
   Do
      If Rx_addresscmd('@unzip -joqq' ecsback_dir || '\krnlmcp.zip -d' ecswork_dir) Then
         If Rx_addresscmd('@attrib -r -h -s' ecswork_dir || '\*.* /s', '-q') Then
         If Rx_repmod(ecswork_dir || '\os2ldr', target_drive || '\os2ldr', '-f') Then
         If Rx_addresscmd('@attrib +r +h +s' target_drive || '\os2ldr', '-q') Then
         If Rx_repmod(ecswork_dir || '\os2krnl', target_drive || '\os2krnl', '-f') Then
         If Rx_addresscmd('@attrib +r +h +s' target_drive || '\os2krnl', '-q') Then
         If Rx_repmod(ecswork_dir || '\sesdd32.sys', target_drive || '\os2\sesdd32.sys', '-f') Then
         If Rx_repmod(ecswork_dir || '\doscall1.dll', target_drive || '\os2\dll\doscall1.dll', '-f') Then
         Do
            Call Rx_sayout 'The eCS runtime modules have been installed on drive' target_drive
            Call Rx_sayout 'Please re-boot drive' target_drive 'to activate'
            upd_desc = ecs_desc
            xrc = 1
         End
      Else
         xrc = 0
   End
Else
   xrc = 0
If \xrc Then
   Call Rx_sayout 'Unable to install the eCS runtime modules'
Return xrc
/*:VRX         Run_fservice
*/
Run_fservice:
/* invoke (F)SERVICE.EXE */

service_bypass = 1
If \(apply | backout) | \system_fix Then
   Return 1

If apply Then
   Call Rx_sayout 'Ready to apply service from IBM Fixpack' valid_fix
Else
   Call Rx_sayout 'Ready to remove service applied by IBM Fixpack' valid_fix
If fixstart_drive = 'ERROR:' Then
   Do
      Call Rx_sayout 'Press "enter" to continue or enter any other response', 1
      If apply Then
         Call Rx_sayout 'to bypass the application of service from IBM Fixpack' valid_fix, 1
      Else
         Call Rx_sayout 'to bypass the removal of service applied by IBM Fixpack' valid_fix, 1
      Parse Upper Pull ans .
   End
Else
   ans = ''
If ans <> '' Then
   Do
      If apply Then
         Call Rx_sayout 'The application of service from IBM Fixpack' valid_fix 'has been bypassed'
      Else
         Call Rx_sayout 'The removal of service applied by IBM Fixpack' valid_fix 'has been bypassed'
      service_bypass = 1
      Return 1
   End

Call Rx_sayout ''
If smp_kernel | install_smp Then
   Do
      If sys_comp <> acp_comp Then
         Do
            Call Rx_sayout 'Updating' sys_lvl 'for SMP service', , 1
            Call Update_syslevel acp_comp
         End
   End
Else
   Do
      If sys_comp <> mcp_comp Then
         Do
            Call Rx_sayout 'Updating' sys_lvl 'for non-SMP service', , 1
            Call Update_syslevel mcp_comp
         End
   End
xrc_set = Value('CSFUTILPATH', srv_path, 'os2environment')
xrc_set = Value('CSFCDROMDIR', csf_path, 'os2environment')
xrc_set = Value('CSFDRIVEAPPLY', target_drive, 'os2environment')
xrc_set = Value('CSFTIMESTAMP', '1', 'os2environment')
service_call = service_exe '/R:' || response_file '/L:' || service_log
/*  '/S:' || csf_path '/B:' || Left(boot_drive, 1) */
If backout Then
   xrc_set = Value('CSFSKIPCRC', '1', 'os2environment')
verbose = 0
Call Rx_sayout 'Invoking' service_exe 'with the following parameters:'
Call Rx_sayout 'Contents of' response_file || ':', 1
Call Rx_readlines response_file
Do x = 1 To file_lines.0
   Call Rx_sayout file_lines.x, 2
End
Call Rx_sayout 'Values of environmental variables:', 1
Call Rx_sayout 'CSFUTILPATH =' Value('CSFUTILPATH', , 'os2environment'), 2
Call Rx_sayout 'CSFCDROMDIR =' Value('CSFCDROMDIR', , 'os2environment'), 2
Call Rx_sayout 'REMOTE_INSTALL_STATE =' Value('REMOTE_INSTALL_STATE', , 'os2environment'), 2
Call Rx_sayout 'CSFDRIVEAPPLY =' Value('CSFDRIVEAPPLY', , 'os2environment'), 2
Call Rx_sayout 'CSFTIMESTAMP =' Value('CSFTIMESTAMP', , 'os2environment'), 2
Call Rx_sayout 'CSFSKIPCRC =' Value('CSFSKIPCRC', , 'os2environment'), 2
Call Rx_sayout 'Value of service call:', 1
Call Rx_sayout 'CALL =' service_call, 2
verbose = 1

Call Rx_addresscmd '@attrib -r' sys_lvl, '-q'
Call SysSleep 3
If apply Then
   Call Rx_sayout 'Applying service from IBM Fixpack' valid_fix '(Kaffee oder Bier Zeit)'
Else
   Call Rx_sayout 'Removing service applied by IBM Fixpack' valid_fix '(Kaffee oder Bier Zeit)'
Call Rx_sayout 'If you hear a "beep" after the IBM service processing has started'
Call Rx_sayout 'Open the "fservice.exe" window and follow the instructions'
Call Rx_addresscmd service_call
If Rx_fileexists(service_log) Then
   Do
      Call Rx_readlines service_log
      z = file_lines.0
      y = z - 1
      If backout Then
         Call Rx_sayout file_lines.y
      Call Rx_sayout file_lines.z
      srv_xrc = ((Pos('successfully', file_lines.y) <> 0) ,
         | (Pos('successfully', file_lines.z) <> 0) ,
         | (Pos('service has completed', file_lines.z) <> 0)) ,
         & (Pos('CSF', file_lines.z) = 0)
   End
Else
   Do
      If apply Then
         Call Rx_sayout 'The application of service from IBM Fixpack' valid_fix 'did not generate a service log'
      Else
         Call Rx_sayout 'The removal of service applied by IBM Fixpack' valid_fix 'did not generate a service log'
      srv_xrc = 0
   End

xrc_set = Value('CSFUTILPATH', '', 'os2environment')
xrc_set = Value('CSFCDROMDIR', '', 'os2environment')
xrc_set = Value('REMOTE_INSTALL_STATE', '', 'os2environment')
xrc_set = Value('CSFDRIVEAPPLY', '', 'os2environment')
xrc_set = Value('CSFTIMESTAMP', '', 'os2environment')
xrc_set = Value('CSFSKIPCRC', '', 'os2environment')

Call Stream sys_lvl, 'c', 'close'
Call Stream sys_lvl, 'c', 'open read'
sys_curr = Charin(sys_lvl, 45, 7)
sys_comp = Charin(sys_lvl, 141, 9)
Call Stream sys_lvl, 'c', 'close'
If sys_comp <> mcp_comp Then
   Do
      Call Rx_sayout 'Reverting' sys_lvl 'to normal values', , 1
      Call Update_syslevel mcp_comp
   End
If srv_xrc Then
   Do
      fix_date_time = Left(Date('S'), 4) || '.' || Substr(Date('S'), 5, 2) || '.' || Substr(Date('S'), 7) Time()
      If apply Then
         Do
            Call Rx_sayout 'The application of service from IBM Fixpack' valid_fix 'has completed succesfully', , 1
            Call SysIni ecsupdt_ini, valid_fix, 'Applied', fix_date_time
            Call SysIni ecsupdt_ini, valid_fix, 'Removed', 'DELETE:'
         End
      Else
         Do
            Call Rx_sayout 'The removal of service applied by IBM Fixpack' valid_fix 'has completed succesfully', , 1
            Call SysIni ecsupdt_ini, valid_fix, 'Applied', 'DELETE:'
            Call SysIni ecsupdt_ini, valid_fix, 'Removed', fix_date_time
         End
      If fixstart_drive <> 'ERROR:' Then
         Call Rx_sayout 'The recovery of service for IBM Fixpack' valid_fix 'processing is complete'
      Do i = 1 To Words(system_updates)
         Call SysIni ecsupdt_ini, Word(system_updates, i), 'DELETE:'
      End
   End
Else
   Do
      If apply Then
         Call Rx_sayout 'The application of service from IBM Fixpack' valid_fix 'may not have completed succesfully'
      Else
         Call Rx_sayout 'The removal of service applied by IBM Fixpack' valid_fix 'may not have completed succesfully'
   End

Call SysIni ecsupdt_ini, 'FIXSTART', 'DELETE:'
local_drives = SysDriveMap(, 'local')
Do i = 1 To Words(local_drives)
   vol_ltr = Word(local_drives, i)
   If SysFileSystemType(vol_ltr) <> 'CDFS' Then
      If Rx_fileexists(vol_ltr || '\fixstart') Then
      Do
         Call SysIni ecsupdt_ini, 'FIXSTART', 'TARGET', target_drive
         Call SysIni ecsupdt_ini, 'FIXSTART', 'SERVICE', apply
         Call SysIni ecsupdt_ini, 'FIXSTART', 'ECSWORK', ecswork_dir
      End
End

service_bypass = 0
xrc = srv_xrc

Return xrc
/*:VRX         Rx_addresscmd
*/
Rx_addresscmd: Procedure Expose (log_parms)
/* address cmd with return code */
save_trace = Trace('N')
Parse Arg cmd_line, opts, .
opts = Translate(opts)
quiet = Abbrev(opts, '-Q')
Call Rx_dequeue
Address cmd cmd_line '2>&1|RXQUEUE'
xrc = (rc = 0)
/* if this was an eject, wait for user response */
If Abbrev(opts, '-RX') Then
   Return xrc
Do Queued()
   Parse Pull ans
   If \quiet Then
      Call Rx_sayout ans
   Else
      Call Rx_sayout ans, , 1
End
Trace(save_trace)
Return xrc

/*:VRX         Rx_compare
*/
Rx_compare: Procedure Expose (log_parms)
save_trace = Trace('N')
/* test exposed parameters */
If \Datatype(verbose, 'b') Then
   verbose = 1
If \Datatype(log_out, 'b') Then
   Do
      xrc = SysFileTree(log_file, 'test.', 'fo')
      If xrc = 0 & test.0 <> 0 Then
         log_out = 1
      Else
         log_out = 0
   End
Parse Upper Arg file1, file2
If Rx_fileexists(file1) & Rx_fileexists(file2) Then
   Do
      Call Rx_dequeue
      Address cmd '@checksum.exe' file1 '2>&1|RXQUEUE'
      Do Queued()
         Parse Pull output
         If Pos('Checksum for file', output) <> 0 Then
            Do
               Parse Var output . '\' . ':' check1 .
               check1 = Space(check1)
            End
      End
      Address cmd '@checksum.exe' file2 '2>&1|RXQUEUE'
      Do Queued()
         Parse Pull output
         If Pos('Checksum for file', output) <> 0 Then
            Do
               Parse Var output . '\' . ':' check2 .
               check2 = Space(check2)
            End
      End
      xrc = (check1 = check2)
   End
Else
   xrc = 0
Trace(save_trace)
Return xrc

/*:VRX         Rx_fileexists
*/
Rx_fileexists: Procedure Expose (log_parms)
save_trace = Trace('N')
Parse Arg file_name
xrc = SysFileTree(file_name, 'test.', 'fo')
If xrc = 0 & test.0 <> 0 Then
   exists = 1
Else
   Do
      /* may be a directory */
      xrc = SysFileTree(file_name, 'test.', 'do')
      If xrc = 0 & test.0 <> 0 Then
         exists = 1
      Else
         exists = 0
   End
Trace(save_trace)
Return exists

/*:VRX         Rx_findlines
*/
Rx_findlines: Procedure Expose (log_parms)
/* find a phrase in a file and return the line number,
   line, and all remmed out lines containing the phrase */
save_trace = Trace('N')
Parse Arg phrase, file, flag
flag = Translate(flag)
first = (Pos('-F', flag) <> 0)
found_line_no = 0
found_line = ''
line_rem_no = ''
If phrase <> '' & Stream(file, 'c', 'query exists') <> '' Then
   Do
      Call Stream file, 'c', 'close'
      xrc = SysFileSearch(phrase, file, 'lines.', 'N')
      If xrc = 0 & lines.0 <> 0 Then
         Do i = 1 To lines.0
            Parse Var lines.i line_no line
            line_test = Strip(Strip(Translate(line), , "'"), , '"')
            If Abbrev(line_test, 'REM') ,
               | Abbrev(line_test, '/*') ,
               | Abbrev(line_test, ';') Then
               Do
                  line_rem_no = line_rem_no line_no
                  Iterate i
               End
            Else
               Do
                  found_line_no = line_no
                  found_line = line
                  If first Then
                     Leave i
               End
         End
   End
Trace(save_trace)
Return found_line_no found_line '|' Strip(line_rem_no)

/*:VRX         Rx_makepath
*/
Rx_makepath: Procedure Expose (log_parms)
save_trace = Trace('N')
Parse Upper Arg path_name
If \Datatype(verbose, 'b') Then
   verbose = 1
If \Datatype(log_out, 'b') Then
   Do
      xrc = SysFileTree(log_file, 'test.', 'fo')
      If xrc = 0 & test.0 <> 0 Then
         log_out = 1
      Else
         log_out = 0
   End
path_name = Strip(path_name, 't', '\')
xrc = SysFileTree(path_name, 'test.', 'do')
If xrc = 0 & test.0 <> 0 Then
   xrc = 1
Else
   Do
      If Pos(':\', path_name) = 2 Then
         Do
            path_name = path_name || '\'
            path. = ''
            path.0 = 0
            path_pos = Pos('\', path_name, 4 )
            Do i = 1 Until path_pos = 0
               path.i = Left(path_name, path_pos - 1)
               path.0 = i
               path_pos = Pos('\', path_name, path_pos + 1)
            End
            Do i = 1 To path.0
               xrc = SysFileTree(path.i, 'test.', 'do')
               If xrc = 0 & test.0 <> 0 Then
                  xrc = 1
               Else
                  Do
                     xrc = SysMkDir(path.i)
                     xrc = (xrc = 0)
                  End
            End
         End
      Else
         xrc = 0
   End
Trace(save_trace)
Return xrc

/*:VRX         Rx_readlines
*/
Rx_readlines: Procedure Expose file_lines. (log_parms)
save_trace = Trace('N')
Parse Arg input_file, flag
If \Datatype(verbose, 'b') Then
   verbose = 1
If \Datatype(log_out, 'b') Then
   Do
      xrc = SysFileTree(log_file, 'test.', 'fo')
      If xrc = 0 & test.0 <> 0 Then
         log_out = 1
      Else
         log_out = 0
   End
flag = Translate(flag)
set_eof = (Pos('-E', flag) <> 0)
If Stream(input_file, 'c', 'query exists') <> '' Then
   Do
      Call Stream input_file, 'c', 'close'
      If set_eof Then
         Do
            /* check for eof character */
            file_size = Stream(input_file, 'c', 'query size')
            Call Stream input_file, 'c', 'open read'
            If Charin(input_file, file_size, 1) = '1a'x Then
               eof = 1
            Else
               eof = 0
            Call Stream input_file, 'c', 'close'
         End
      Else
         eof = 0
      Call Stream input_file, 'c', 'open read'
      file_lines. = ''
      file_lines.0 = 0
      Do i = 1 Until Lines(input_file) = 0
         file_lines.i = Linein(input_file)
      End
      file_lines.0 = i
      If eof Then
         Do
            i = file_lines.0 + 1
            file_lines.i = '1a'x
            file_lines.0 = i
         End
      Call Stream input_file, 'c', 'close'
      xrc = 1
   End
Else
   xrc = 0
Trace(save_trace)
Return xrc

/*:VRX         Rx_repmod
*/
Rx_repmod: Procedure Expose (log_parms)
save_trace = Trace('N')
/* test exposed parameters */
If \Datatype(verbose, 'b') Then
   verbose = 1
If \Datatype(log_out, 'b') Then
   Do
      xrc = SysFileTree(log_file, 'test.', 'fo')
      If xrc = 0 & test.0 <> 0 Then
         log_out = 1
      Else
         log_out = 0
   End
Parse Arg source_file, target_file, opts, .
force = 0
backup = 0
quiet = 0
If opts <> '' Then
   Do
      Parse Var opts flags backup_file .
      flags = Space(Translate(flags))
      force = (Pos('F', flags) <> 0)
      backup = (Pos('B', flags) <> 0)
      quiet = (Pos('Q', flags) <> 0)
      If backup Then
         Do
            backup_file = Space(backup_file)
            If backup_file = '' Then
               Do
                  Call Rx_sayout 'Backup was specified, but no backup file was given'
                  Call Rx_sayout 'No backup of' target_file 'will be made', 1
                  backup = 0
               End
         End
   End
Call Rx_dequeue
xrc = SysFileTree(source_file, 'source.', 'fo')
If xrc = 0 & source.0 <> 0 Then
   Do
      target_path = Strip(Filespec('d', target_file) || Filespec('p', target_file), 't', '\')
      xrc = SysFileTree(target_path, 'target.', 'do')
      If xrc = 0 & target.0 <> 0 Then            /* found target path */
         Do
            rep_it = 0
            x_crc = Rx_compare(source_file, target_file)
            Select
               When force Then
                  rep_it = 1
               When \x_crc Then
                  rep_it = 1
               When x_crc Then
                  Do
                     If \quiet Then
                        Do
                           Call Rx_sayout 'The check sum of the target file'
                           Call Rx_sayout 'Target file -' target_file, 1
                           Call Rx_sayout 'is equal to the check sum the source file'
                           Call Rx_sayout 'Source_file -' source_file, 1
                           Call Rx_sayout 'Enter "go" to overwrite the target file'
                           Call Rx_sayout 'or enter any other response to bypass the update', 1
                           Parse Upper Pull ans .
                           If ans = 'GO' Then
                              rep_it = 1
                           Else
                              rep_it = 0
                        End
                     Else
                        rep_it = 0
                  End
               Otherwise
                  Do
                     Call Rx_sayout 'An unexpected condition has occured while processing'
                     Call Rx_sayout 'Target file -' target_file '-' target_stamp '-' target_size, 1
                     Call Rx_sayout 'Source file -' source_file '-' source_stamp '-' source_size, 1
                     Call Rx_sayout 'Update of' target_file 'bypassed'
                     Return 0
                  End
            End
            If rep_it Then
               Do
                  If backup Then
                     Do
                        xrc = SysFileTree(target_file, 'target.', 'fo')
                        If xrc = 0 & target.0 <> 0 Then
                           Do
                              Address cmd 'zip -oSX9' backup_file target_file '2>&1|RXQUEUE'
                              Call Rx_dequeue
                           End
                        Else
                           Do
                              Call Rx_sayout target_file 'does not currently', 1
                              Call Rx_sayout 'exist and will not be backed up', 2
                           End
                     End
                  Call Rx_dequeue
                  Address cmd '@repmod' source_file target_file '2>&1|RXQUEUE'
                  xrc = (rc = 0)
                  If quiet Then
                     verbose = 0
                  Do Queued()
                     Parse Pull ans
                     Call Rx_sayout ans, 1
                  End
                  verbose = 1
               End
            Else
               Do
                  If quiet Then
                     verbose = 0
                  Call Rx_sayout 'Bypassing replacement of' target_file
                  verbose = 1
               End
         End
      Else
         Do
            Call Rx_sayout 'Target path' target_path 'not found'
            Return 0
         End
   End
Else
   Do
      Call Rx_sayout 'Source' source_file 'not found'
      Return 0
   End
Trace(save_trace)
Return xrc

/*:VRX         Rx_sayout
*/
Rx_sayout: Procedure Expose (log_parms)
/* console and log file output */
save_trace = Trace('N')
/* test exposed parameters */
If \Datatype(verbose, 'b') Then
   verbose = 1
If \Datatype(log_out, 'b') Then
   Do
      xrc = SysFileTree(log_file, 'test.', 'fo')
      If xrc = 0 & test.0 <> 0 Then
         log_out = 1
      Else
         log_out = 0
   End
Parse Arg msg, level, log_parm
If \Datatype(level, 'W') Then
   level = 0
Parse Var log_parm
If \Datatype(log_parm, 'W') Then
   log_parm = 0
log_only = (log_parm > 0)
no_log = (log_parm < 0)
msg = Copies(' ', level) || msg
If verbose & \log_only Then
   Say msg
If \no_log Then
   Do
      If log_out Then
         Call Lineout log_file, Date() Time() msg
      Else
         Do
            x = log_lines.0 + 1
            log_lines.x = level log_parm Strip(msg)
            log_lines.0 = x
         End
   End
Trace(save_trace)
Return level

/*:VRX         Rx_searchlibpath
*/
Rx_searchlibpath: Procedure
Parse Arg dll
config_sys = SysBootDrive() || '\config.sys'
xrc = SysFileSearch('libpath', config_sys, 'lib_path.')
If xrc = 0 & lib_path.0 <> 0 Then
   Do i = 1 To lib_path.0
      lib_test = Space(Translate(lib_path.i))
      If Abbrev(lib_test, 'REM') Then
         Iterate i
      If \Abbrev(lib_test, 'LIBPATH') Then
         Iterate i
      lib_equal = Space(Substr(lib_test, 8))
      If \Abbrev(lib_equal, '=') Then
         Iterate i
      Parse Var lib_test . '=' lib_path
   End
lib_path = Space(lib_path)
temp_path = SysTempFileName('lib????')
old_value = Value(temp_path, lib_path, 'os2environment')
dll_path = SysSearchPath(temp_path, dll)
old_value = Value(temp_path, old_value, 'os2environment')
Return dll_path

/*:VRX         Rx_updtline
*/
Rx_updtline: Procedure Expose file_lines. (log_parms)
save_trace = Trace('N')
Parse Arg line_no, output_line, output_file, action, opts
If \Datatype(verbose, 'b') Then
   verbose = 1
If \Datatype(log_out, 'b') Then
   Do
      xrc = SysFileTree(log_file, 'test.', 'fo')
      If xrc = 0 & test.0 <> 0 Then
         log_out = 1
      Else
         log_out = 0
   End
xrc = 1
replace = 0
insert = Abbrev(Translate(action), 'I')
If \insert Then
   replace = 1
opts = Translate(opts)
close = (Pos('C', opts) <> 0)
quiet = (Pos('Q', opts) <> 0)
eof = (Pos('E', opts) <> 0)

If file_lines.0 = 0 Then
   Do
      If \eof Then
         Call Rx_readlines output_file
      Else
         Call Rx_readlines output_file, '-e'
   End

xrc = 1
x = file_lines.0                           /* number of lines in file */
line_no = Format(line_no)
If replace Then
   Do                                     /* replace an existing line */
      If file_lines.x = '1a'x ,                  /* don't replace eof */
         & line_no = x Then
         xrc = 0
      If xrc Then
         file_lines.line_no = output_line
      Else
         Call Rx_sayout 'Replacement of end-of-file marker not permitted'
   End
Else                                             /* insert a new line */
   Do
      If line_no = 0 Then                     /* insert as first line */
         line_no = 1
      If line_no > file_lines.0 Then           /* insert as last line */
         line_no = file_lines.0
      If line_no < 0 Then                      /* do no insert a line */
         line_no = 0
      If line_no <> 0 Then
         xrc = \SysStemInsert('file_lines.', line_no, output_line)
      Else
         xrc = 1
   End

If xrc Then
   Do
      If close Then
         Do
            xrc = 0
            save_file = SysTempFileName(Left(output_file, Lastpos('.', output_file) - 1) || '.???')
            If \quiet Then
               Do
                  Call Rx_sayout ''
                  Call Rx_sayout 'Ready to rewrite' output_file || ', enter OK to continue', 1
                  Parse Upper Pull ans .
               End
            Else
               ans = 'OK'
            If ans = 'OK' Then
               Do
                  Address cmd '@copy' output_file save_file '> nul'
                  xrc = (rc = 0)
                  If xrc Then
                     Do
                        Address cmd '@erase' output_file
                        If rc = 0 Then
                           Do
                              Call Rx_writelines output_file
                              xrc = 1
                              Call Rx_sayout 'Updated' output_file || ', was saved as' save_file, 1, quiet
                           End
                        Else
                           Do
                              xrc = 0
                              Call Rx_sayout 'Deletion of' output_file 'failed with a return code of' rc, 1
                              Call Rx_sayout 'File' output_file 'was saved as' save_file, 2
                           End
                     End
                  Else
                     Do
                        xrc = 0
                        Call Rx_sayout 'Copy of' output_file 'to' save_file, 1
                        Call Rx_sayout 'failed with a return code of' rc, 2
                        Call Rx_sayout output_file 'should be intact', 2
                     End
               End
            Else
               Do
                  xrc = 0
                  Call Rx_sayout 'Update of' output_file 'aborted', 1
               End
            file_lines. = ''
            file_lines.0 = 0
         End
   End
Trace(save_trace)
Return xrc

/*:VRX         Rx_writelines
*/
Rx_writelines: Procedure Expose file_lines. (log_parms)
save_trace = Trace('N')
Parse Arg output_file
Call Stream output_file, 'c', 'close'
Call Stream output_file, 'c', 'open write'
x = file_lines.0
If file_lines.x = '1a'x Then
   eof = 1
Else
   eof = 0
chars_out = 0
Do i = 1 To file_lines.0
   If \eof Then
      Call Lineout output_file, file_lines.i
   Else
      Do
         If eof & i <> (file_lines.0) Then
            Do
               Call Lineout output_file, file_lines.i
               chars_out = chars_out + Length(file_lines.i) + 2
            End
         Else
            Call Charout output_file, '1a'x, (chars_out + 1)
      End
End
Call Stream output_file, 'c', 'close'
Trace(save_trace)
Return 1

/*:VRX         Select_fixpack
*/
Select_fixpack:
/* select a fixpack for this run */
save_trace = Trace('N')
valid_fixes. = ''
valid_fixes.0 = 0
Do i = 1 To valid.0
   Parse Var valid.i valid_sys valid_csf valid_data
   valid_sys = Strip(Translate(valid_sys))
   valid_csf = Strip(Translate(valid_csf))
   valid_data = Space(valid_data)
   If Abbrev(valid_csf, 'CSF') Then
      Do
         If lock_exists ,
            | \Rx_fileexists(our_path || valid_csf || '.ZIP') Then
            Iterate i
         Parse Var valid_data . . . . valid_lvl ',' .
         valid_lvl = Space(valid_lvl)
         sys_lvl = target_drive || valid_sys
         sys_base = Charin(sys_lvl, 53, 7)
         Call Stream sys_lvl, 'c', 'close'
         sys_base_tst = Substr(sys_base, 1, 2) || '_' || Substr(sys_base, 4)
         If Wordpos(sys_base_tst, valid_lvl) <> 0 Then
            Do
               j = valid_fixes.0 + 1
               valid_fixes.j = sys_lvl valid.i
               valid_fixes.0 = j
            End
         Else
            Iterate i
      End
   Else
      Do
         If \Rx_fileexists(our_path || Space(valid_csf)) Then
            Iterate i
         If Abbrev(valid_csf, 'SMP') & \ecs_pro Then
            Iterate i
         Parse Var valid_data '|' valid_fix '|' valid_no_uni valid_no_smp . valid_lvl ',' .
         valid_fix = Space(valid_fix)
         valid_lvl = Space(valid_lvl)
         If \Abbrev(valid_fix, '\\') & \Rx_fileexists(our_path || valid_fix) Then
            Iterate i
         If Abbrev(valid_csf, 'JRT') & target_drive = boot_drive Then
            Iterate i
         sys_lvl_file = ''
         /* search path first */
         Parse Value(Rx_findlines('set path=', config_sys)) With ,
            path_line_no path_line '|' .
         If path_line_no <> 0 Then
            Do
               valid_sys_path = Left(valid_sys, Lastpos('\', valid_sys) -1)
               valid_sys_name = Substr(valid_sys, Lastpos('\', valid_sys) + 1)
               Parse Var path_line . '=' path_data
               path_data = Strip(path_data)
               If Lastpos(';', path_data) <> Length(path_data) Then
                  path_data = path_data || ';'
               valid_sys_path_pos = Pos(valid_sys_path || ';', Translate(path_data))
               If valid_sys_path_pos <> 0 Then
                  path_data = Substr(path_data, valid_sys_path_pos - 2)
               Else
                  path_data = target_drive || valid_sys_path || ';'
               Call Value 'ecsapar_path', path_data, 'os2environment'
               sys_lvl_file = SysSearchPath('ecsapar_path', valid_sys_name)
               If sys_lvl_file <> '' Then
                  sys_lvl = sys_lvl_file
            End
         /* not in path, search the target drive */
         If sys_lvl_file = '' Then
            Do
               sys_lvl_file = target_drive || valid_sys
               xrc = SysFileTree(sys_lvl_file, 'sys_lvl.', 'fso')
               If xrc = 0 & sys_lvl.0 <> 0 Then
                  sys_lvl = sys_lvl.1
               Else
                  Iterate i
            End
         sys_curr = Charin(sys_lvl, 45, 7)
         Call Stream sys_lvl, 'c', 'close'
         sys_base_tst = Substr(sys_base, 1, 2) || '_' || Substr(sys_base, 4)
         If Wordpos(sys_curr, valid_lvl) <> 0 Then
            Do
               If Abbrev(valid_csf, 'SMP') & \smp_kernel Then
                  Iterate i
               If Abbrev(valid_csf, 'W4') & smp_kernel Then
                  Iterate i
               j = valid_fixes.0 + 1
               valid_fixes.j = sys_lvl valid.i
               valid_fixes.0 = j
            End
         Else
            Iterate i
      End
End

/* set up recovery mode */
If fixstart_drive <> 'ERROR:' Then
   valid_fixes.0 = 1

If valid_fixes.0 <> 0 Then
   Do
      If valid_fixes.0 <> 1 Then
         Do
            Call Rx_sayout 'Please select an update for processing on drive' target_drive, , -1
            leading = Length(valid_fixes.0)
            Do i = 1 To valid_fixes.0
               Parse Var valid_fixes.i sys_lvl valid_sys valid_csf valid_data
               If Abbrev(valid_csf, 'CSF') Then
                  Parse Var valid_data valid_fix valid_no_uni valid_no_smp valid_free valid_lvl ',' valid_desc
               Else
                  Parse Var valid_data '|' valid_fix '|' valid_no_uni valid_no_smp valid_free valid_lvl ',' valid_desc
               valid_lvl = Space(valid_lvl)
               valid_desc = Strip(valid_desc)
               fix_status_text = ''
               If \Abbrev(valid_csf, 'CSF') Then
                  Do
                     Parse Var valid_csf valid_csf_name '.' .
                     valid_csf_name = Translate(Strip(valid_csf_name))
                     fix_status_text = SysIni(ecsupdt_ini, valid_csf_name, Translate(valid_csf))
                     If fix_status_text = 'ERROR:' Then
                        fix_status_text = SysIni(ecsupdt_ini, valid_csf_name, valid_csf_name || '.CDR')
                     If fix_status_text <> 'ERROR:' Then
                        fix_status_text = 'Applied' fix_status_text
                     Else
                        Do
                           fix_status_text = SysIni(ecsupdt_ini, valid_csf_name, valid_csf_name || '.BOL')
                           If fix_status_text <> 'ERROR:' Then
                              fix_status_text = 'Backed out' fix_status_text
                           Else
                              fix_status_text = ''
                        End
                     backup_name = ptfback_dir || '\' || valid_csf_name || '.BAK'
                     If Rx_fileexists(backup_name) & fix_status_text <> '' Then
                        fix_status_text = fix_status_text || ', backout available'
                     If Rx_fileexists(backup_name) & fix_status_text = '' Then
                        fix_status_text = 'backout available'
                  End
               Else
                  Do
                     fix_status_text = SysIni(ecsupdt_ini, valid_fix, 'Applied')
                     If fix_status_text <> 'ERROR:' Then
                        fix_status_text = 'Applied' fix_status_text
                     Else
                        Do
                           fix_status_text = SysIni(ecsupdt_ini, valid_fix, 'Removed')
                           If fix_status_text <> 'ERROR:' Then
                              fix_status_text = 'Removed' fix_status_text
                           Else
                              Do
                                 If Translate(valid_fix, '0', '_') = sys_curr Then
                                    Do
                                       fix_date_time = Translate(SysGetFileDateTime(target_drive || '\os2\install\service.log', 'W'), '.', '-')
                                       Call SysIni ecsupdt_ini, valid_fix, 'Applied', fix_date_time
                                       fix_status_text = 'Applied' fix_date_time
                                    End
                                 Else
                                    fix_status_text = ''
                              End
                        End
                  End
               If fix_status_text <> '' Then
                  fix_status_text = '(' || fix_status_text || ')'
               Call Rx_sayout Right(i, leading) '-' Left(Word(valid_desc, 1), 8) Subword(valid_desc, 2), , -1
               If fix_status_text <> '' Then
                  Call Rx_sayout fix_status_text, Value(leading + 5), -1
            End
            Call Rx_sayout 'Any invalid selection will exit', Value(leading + 3), -1
            Parse Upper Pull ans .
         End
      Else
         ans = 1
      If ans > 0 & ans <= valid.0 Then
         Do
            system_fix = 0
            Parse Var valid_fixes.ans sys_lvl valid_sys valid_csf valid_data
            valid_csf = Translate(valid_csf)
            If Abbrev(valid_csf, 'CSF') Then
               Do
                  Parse Var valid_data valid_fix valid_no_uni valid_no_smp valid_free valid_lvl ',' valid_desc
                  system_fix = 1
               End
            Else
               Parse Var valid_data '|' valid_fix '|' valid_no_uni valid_no_smp valid_free valid_lvl ',' valid_desc
            valid_fix = Space(valid_fix)
            valid_lvl = Space(valid_lvl)
            valid_desc = Strip(valid_desc)
            java_fix = Abbrev(valid_csf, 'JRT')
            If \java_fix Then
               work_free = valid_free
            Else
               work_free = 0
            xrc = 1
         End
      Else
         Do
            Call Rx_sayout 'No update selected, exiting', , -1
            xrc = 0
         End
   End
Else
   Do
      Call Rx_sayout 'No valid updates were found'
      xrc = 0
   End
Trace(save_trace)
Return xrc

/*:VRX         Update_config_sys
*/
Update_config_sys:

If \smp_upgrade Then
   Return 1

psd_sys = 'PSD=OS2APIC.PSD'
config_update_line_no = ''
config_update_line = ''
config_action = ''
config_locked = 0

/* locate os2apic.psd line in config.sys */
Parse Value(Rx_findlines(psd_sys, config_sys)) With ,
   psd_line_no psd_line '|' psd_line_rem
/* setup the update for config.sys */
Select
   When install_smp & psd_line_no = 0 & psd_line_rem = '' Then
      Do                         /* add psd=os2apic.psd to config.sys */
         config_update_line_no = 0
         config_update_line = psd_sys
         config_action = 'insert'
      End
   When install_smp & psd_line_rem <> '' Then
      Do                      /* un-REM psd=os2apic.psd in config.sys */
         config_update_line_no = Word(psd_line_rem, 1)
         config_update_line = psd_sys
         config_action = 'replace'
      End
   When uninstall_smp & psd_line_no <> 0 Then
      Do                         /* REM psd=os2apic.psd in config.sys */
         config_update_line_no = psd_line_no
         config_update_line = 'REM' psd_line
         config_action = 'replace'
      End
   Otherwise Nop
End

/* update config.sys */
If config_update_line <> '' Then
   Do
      If target_drive = boot_drive Then   /* partition being serviced */
         config_locked = 1                 /* is the booted partition */
      config_sys_target = config_sys
      file_lines.0 = 0
      xrc = Rx_updtline(config_update_line_no, config_update_line, ,
         config_sys_target, config_action, '-cq')
      If xrc & config_locked & install_smp Then
         Do
            Call Rx_sayout 'The addition of the line' config_update_line 'to' config_sys_target
            Call Rx_sayout 'may give an error at your next re-boot', 1
            Call Rx_sayout 'You may ignore the error'
         End
   End

If xrc Then
   Do
      If upd_desc <> sys_desc Then
         xrc = Update_syslevel(, upd_desc)
      Else
         xrc = 1
      sys_desc = Strip(Translate(Charin(sys_lvl, 61, 80), ' ', '00'x), 't')
      Call Stream sys_lvl, 'c', 'close'
   End

Return xrc

/*:VRX         Update_smp
*/
Update_smp:
/* handle smp updates */

If \smp_upgrade Then
   Return 1

If install_smp Then
   Do
      If Rx_fileexists(our_path || 'os2root\os2\boot\os2apic.psd') Then
         Do
            If Get_smp_runtime() Then
               Do
                  If Backup_ecs_code() Then
                     Do
                        If Load_smp_code() Then
                           xrc = 1
                        Else
                           xrc = 0
                     End
                  Else
                     xrc = 0
               End
            Else
               xrc = 0
         End
      Else
         Do
            Call Rx_sayout 'Unable to locate the SMP support on your eCS Installation CD'
            Call Rx_sayout 'SMP support will not be added to drive' boot_drive, 1
            xrc = 0
         End
   End
Else                                                 /* uninstall smp */
   Do
      If Check_ecs_backup() Then
         Do
            If Prepare_ecs_backup_target() Then
               Do
                  If Restore_ecs_backup() Then
                     xrc = 1
                  Else
                     xrc = 0
               End
            Else
               xrc = 0
         End
      Else
         xrc = 0
   End
/* invalidate any kernel updates */
If xrc Then
   Do i = 1 To Words(kernel_updates)
      Call SysIni ecsupdt_ini, Word(kernel_updates, i), 'DELETE:'
   End

Return xrc
/*:VRX         Update_syslevel
*/
Update_syslevel: Procedure Expose (log_parms) sys_lvl system_fix
/* update syslevel file */

If \system_fix Then
   Return 1

Parse Arg upd_comp, upd_desc
upd_desc = Space(upd_desc)
sys_lvl_upd = Filespec('d', sys_lvl) ,
   || Filespec('p', sys_lvl) || 'SYSLEVEL.UPD'
sys_lvl_org = Filespec('d', sys_lvl) ,
   || Filespec('p', sys_lvl) || 'SYSLEVEL.ORG'
sys_lvl_sav = Filespec('d', sys_lvl) ,
   || Filespec('p', sys_lvl) || 'SYSLEVEL.SAV'
Call Rx_addresscmd '@attrib -r' sys_lvl, '-q'
/* save eCS installation syslevel.os2 */
If system_fix Then
   Do
      If \Rx_fileexists(sys_lvl_org) Then
         xrc = Rx_repmod(sys_lvl, sys_lvl_org, '-fq')
   End
/* save current target syslevel */
If Rx_fileexists(sys_lvl_sav) Then
   Call SysFileDelete sys_lvl_sav
xrc = Rx_repmod(sys_lvl, sys_lvl_sav, '-fq')
If xrc Then
   Do
      /* read original syslevel file */
      sys_lvl_length = Stream(sys_lvl, 'c', 'query size')
      sys_lvl_org = Charin(sys_lvl, 1, sys_lvl_length)
      Call Stream sys_lvl, 'c', 'close'
      /* build syslevel update file */
      If upd_desc <> '' Then
         sys_lvl_new = Left(sys_lvl_org, 60) || Left(upd_desc, 80, '00'x)
      Else
         sys_lvl_new = Left(sys_lvl_org, 140)
      If upd_comp <> '' Then
         sys_lvl_new = sys_lvl_new || upd_comp || Substr(sys_lvl_org, 150)
      Else
         sys_lvl_new = sys_lvl_new || Substr(sys_lvl_org, 141)
      If Rx_fileexists(sys_lvl_upd) Then
         Call SysFileDelete sys_lvl_upd
      Call Stream sys_lvl_upd, 'c', 'open write'
      xrc = Charout(sys_lvl_upd, sys_lvl_new, 1)
      xrc = (xrc = 0)
      Call Stream sys_lvl_upd, 'c', 'close'

      /* replace syslevel file */
      If xrc Then
         Do
            xrc = Rx_repmod(sys_lvl_upd, sys_lvl, '-fq')
            If xrc Then
               Do
                  Call SysFileDelete sys_lvl_upd
                  Call SysFileDelete sys_lvl_sav
               End
            Else
               Do
                  Call Rx_sayout 'Copying' sys_lvl_sav 'back to' sys_lvl
                  xrc = Rx_repmod(sys_lvl_sav, sys_lvl)
                  If xrc Then
                     Do
                        Call SysFileDelete sys_lvl_upd
                        Call SysFileDelete sys_lvl_sav
                     End
                  Else
                     Do
                        Call Rx_sayout 'Unable to restore' sys_lvl_sav 'to' sys_lvl
                        Call SysFileDelete sys_lvl_upd
                     End
                  xrc = 0
               End
         End
      Else
         Do
            Call Rx_sayout 'Unable to create' sys_lvl_upd
            Call SysFileDelete sys_lvl_sav
            xrc = 0
         End
   End
Else
   Do
      Call Rx_sayout 'Unable to save' sys_lvl
      xrc = 0
   End

Return xrc

/*:VRX         Rx_dequeue
*/
Rx_dequeue: Procedure
Do Queued()
   Pull .
End
Return 1

/*:VRX         rx_driveinfo
Usage:  rx_driveinfo(drive, option)
Where:     drive = drive to be queried
           option = -fr - return free space on a drive
                    -fs - return file system of drive
                    -vn - return volume name of drive
Result:    dependent upon option
*/
Rx_driveinfo: Procedure
save_trace = Trace('N')
Parse Upper Arg drive, opt
drive = Left(drive, 1) || ':'
fr = Abbrev(opt, '-FR')                                 /* free space */
vn = Abbrev(opt, '-VN')                                /* volume name */

Select
   When fr Then
      Do
         Numeric Digits 12
         Parse Value(SysDriveInfo(drive)) With . free .
         If \Datatype(free, 'N') Then
            lvm_ans = 0
         Else
            lvm_ans = Format(free / (1024 * 1000), ,0)
      End
   When vn Then
      Do
         If SysFileSystemType(drive) = 'CDFS' Then
            Do
               Parse Value SysDriveInfo(drive) With . . . vol_label
               lvm_ans = Strip(vol_label)
            End
         Else
            Do
               Call Rx_dequeue
               Address cmd '@lvm /query : volumes , ALL ; 2>&1|RXQUEUE'
               Do i = 1 To Queued()
                  Parse Pull lvm_info
                  Parse Var lvm_info test_vol lvm_name .
                  If test_vol = drive Then
                     Do
                        lvm_ans = Strip(lvm_name)
                        Leave i
                     End
               End
               Call Rx_dequeue
            End
      End
   Otherwise
      Do
         lvm_ans = ''
         If fr Then
            lvm_ans = 0
      End
End
Trace(save_trace)
Return lvm_ans

/* clean out the temporary lock lists */
Clean_locklist: Procedure
Parse Arg target_drive
xrc = SysFileTree(target_drive || '\os2\install\ibmcsflk.*', 'locks.', 'fo')
If xrc = 0 Then
   Do x = 1 To locks.0
      Parse Var locks.x . '.' lock_ext .
      If Datatype(Strip(lock_ext), 'W') Then
         Call SysFileDelete locks.x
   End
Return 0

/* check the lock list */
Find_locklist: Procedure Expose (log_parms)
save_trace = Trace('N')
Parse Upper Arg target_drive .
lock_list = target_drive || '\os2\install\ibmcsflk.lst'
If Rx_fileexists(lock_list) Then
   Do
      lock_exists = 1
      Parse Value(Rx_findlines('RMTREE', lock_list)) With ,
         rmtree_line_no rmtree_line '|' .
      If rmtree_line_no <> 0 Then
         Parse Var rmtree_line . lock_dir .
      Else
         lock_dir = ''
   End
Else
   Do
      lock_exists = 0
      lock_list = ''
      lock_dir = ''
   End
Trace(save_trace)
Return lock_exists lock_list lock_dir

/* make ibmcsflk unique */
Update_ibmcsflk: Procedure Expose (log_parms) file_lines.
save_trace = Trace('N')
Parse Upper Arg target_drive .
xrc = 1
Parse Value Find_locklist(target_drive) With lock_exists lock_list lock_dir .
If lock_exists ,
   & lock_dir <> '' ,
   & Filespec('n', lock_dir) = 'IBMCSFLK' Then
   Do
      lock_drive = Filespec('d', lock_dir)
      new_lock_dir = SysTempFileName(lock_drive || '\IBMCSF??')
      /* update lock list */
      xrc = SysFileSearch(lock_dir, lock_list, 'locks.', 'n')
      If xrc = 0 & locks.0 <> 0 Then
         Do
            Do i = 1 To locks.0
               Parse Var locks.i line_no line
               lock_start = Pos(lock_dir, line)
               If lock_start <> 0 Then
                  Do
                     first_part = lock_start - 1
                     last_part = lock_start + Length(lock_dir)
                     locks.i = line_no ,
                        Substr(line, 1, first_part) ,
                        || new_lock_dir ,
                        || Substr(line, last_part)
                  End
            End
            /* update the lock list */
            file_lines.0 = 0
            line_opt = '-qe'
            Do i = 1 To locks.0
               Parse Var locks.i line_no line
               If i = locks.0 Then
                  line_opt = '-cqe'
               xrc = Rx_updtline(line_no, line, lock_list, 'replace', line_opt)
            End
            /* rename lock directory */
            If xrc Then
               xrc = Rx_addresscmd('rename' lock_dir Filespec('n', new_lock_dir))
         End
   End
Trace(save_trace)
Return xrc

/* CONVERT.CMD now called themeupd.cmd */
Themeupd: Procedure
save_trace = Trace('N')
Parse Arg workdir

/* make work directory current */
save_dir = Directory()
new_dir = Directory(workdir)

/* strips the resources which are going to be replaced */
bmp.0 = 15
bmp.1 = 25
bmp.2 = 26
bmp.3 = 27
bmp.4 = 28
bmp.5 = 29
bmp.6 = 30
bmp.7 = 31
bmp.8 = 32
bmp.9 = 33
bmp.10 = 34
bmp.11 = 48
bmp.12 = 50
bmp.13 = 51
bmp.14 = 52
bmp.15 = 53

Do i = 1 To bmp.0
   Call Resmgrn '-d pmmrgres.dll' bmp.i || '.bitmap'
End

/* creates all the DLLs used by THEMEMGR.EXE */
res.0 = 11
res.1 = ecs
res.2 = ecsmath
res.3 = ecswarp
res.4 = ecswarp2
res.5 = ecswin
res.6 = flat
res.7 = warp3
res.8 = warp4
res.9 = warpal
res.10 = win
res.11 = warpslv

Do i = 1 To res.0
   Address cmd '@copy pmmrgres.dll' res.i || '.dll > nul'
   Call Resmgrn '-a' res.i || '.dll' res.i || '.res'
End

/* restore the caller's directory */
Call Directory save_dir
Trace(save_trace)
Return
